/*      file     : Planner/dijkstra.cc
        author   : Riko Jacob
        RCS      : $Id: heap.cc,v 1.4 2005/03/25 13:41:08 mr Exp $

        Copyright (C) 2005 Riko Jacob and Marcus Rickert
*/

// a priority queue, implemented as a heap, stored in arrays
// objects stored are pointers,
// values are floats
// both number of Elements and member numbers may not exceed size
// size is static, we know this number on creation and don't change it. 


// this one tries to use first level cache as efficient as possible
// doesn't improve things too significant
template<class OBJ>
class tabmemb
  {
  public:
    float  value;
    OBJ member;
  };

template<class OBJECT>
class p_queue_heap
{
  //typedef tabmemb<OBJECT> mytabmemb;

public:
  p_queue_heap( int s=10000 ) : max_size(s), size(0), max_used (0)
    { table  = new tabmemb<OBJECT>[max_size];
    }
  ~p_queue_heap() 
  { 
    delete table; 
    //    cout << "Maximum used heap-size: " << max_used << endl;
  }


  void insert(OBJECT new_member, float new_value); 

  float  min_dist() { return table[0].value;}
  OBJECT min_member() {return table[0].member;}
  OBJECT rem_last();
  void   remove_min();
  void   clear() { size =0; max_used=0; };

  int empty() {return size ==  0;};
  
  //private:

  int test_heap()
  {
    int i,backa=0;
    for( i = size -1; i>0 ; i-- ) 
      if ( !(table[i].value >= table[(i-1)/2].value ) ) 
	{
	  cout <<"<"<< i << ">";
	  backa=1;
	}
    if (backa) 
      {
	cout << endl << "heap property error" <<endl;
	for(i =0; i<size;i++) cout << i <<"=" << table[i].value<<"  ";
	cout << endl;
      }
    return backa;
  };

  int get_size() { return size; };
  
private:


  const int max_size;
  int    size;
  int    first_free;
  int    max_used;
  class tabmemb<OBJECT> *table;
};

template<class OBJECT>
void p_queue_heap<OBJECT>::remove_min()
{
  int pos, newa, newb; // new subheaps

  if ( size <= 0 ) exit(6);  // heap underflow


  size--;

  OBJECT act_member  = table[size].member;
  float  act_value = table[size].value;   

  pos = 0;  // start heapify
  //  while( (newa=2*pos+1) < size  )  // try with <<
  while( (newa=(pos<<1|1)) < size  )  // try with <<
    {
      newb=newa+1;
      if ( newb == size )      // special thing at end of heap
	{
// 	  cout << "heap special " <<pos<<"/"<<newa<<"-"<<newb<<" < " <<size<< endl;
	  if( act_value > table[newa].value ) 
	    {
	      table[pos].member=table[newa].member;
	      table[pos].value =table[newa].value;
	      pos = newa;
	    }
	  break;
	}
      // everything heapified
      //      cout << "heap test" << endl;
      if ( table[newa].value >= act_value && table[newb].value >= act_value )
	{
	  //	  cout << "done"<<endl;
	  break; 
	}
      //cout << "go on" << endl;
      if ( table[newa].value < table[newb].value )  // keep on going the right way
	{
	  table[pos].member=table[newa].member;
	  table[pos].value =table[newa].value;
	  pos=newa;
	}
      else
	{
	  table[pos].member=table[newb].member;
	  table[pos].value =table[newb].value;
	  pos=newb;
	}
    }
  table[pos].member=act_member;
  table[pos].value =act_value;
  //   if( test_heap() ) cout <<"heap error after rem_min" << endl, exit(91);
  return;
}

template<class OBJECT>
void p_queue_heap<OBJECT>::insert(OBJECT new_member, float new_value)
{
  int i = size;  // start at leaf
  int parent;

  if ( size >= max_size ) exit(35);  // bail out: heap overflow
  i = size;
  size++;
  if ( size > max_used ) max_used = size;

  parent=(i-1)/2;
  while( i>0 &&  table[parent].value > new_value  ) 
    {
      table[i].value = table[parent].value;
      table[i].member = table[parent].member;
      i = parent; parent = (i-1)/2;
    }
  table[i].value = new_value;
  table[i].member = new_member;  

//   if( test_heap() ) cout <<"heap error after insert/update" << endl, exit(91);
  return;
}

template<class OBJECT>
OBJECT p_queue_heap<OBJECT>::rem_last()
{
  if ( size == 0 ) exit(93);  // heap underflow
  size--;
  return table[size].member;
}
