/*      file     : RouteConverter/TRouteConverter.h
        project  : TRANSIMS
        authors  : Marcus Rickert
        RCS      : $Id: TRouteConverter.h,v 1.5 2005/03/25 12:28:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TROUTECONVERTER_H
#define TROUTECONVERTER_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectHandler.h>

// ***FORWARDS************************************************************************

class TEnvironment;
class TBaseDataManager;
class TObjectManager;
class TParser;
class TTrafficDataManager;
class TRandom;
class TRouter;
class TShortestPath;
class TString;
class TStream;
class TSynchronize;
class TTimeStep;
class TVehicle;

#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***CLASSES************************************************************************* 

class TRouteConverter : public TBaseObject, public TObjectHandler

{
 public:

    TRouteConverter(void);
    virtual ~TRouteConverter(void);

    bool
    Start(int argc, char *argv[]);

 private:

    TEnvironment         *Environment;
    TObjectManager       *ObjectManager;
    TBaseDataManager     *BaseDataManager;
    TTrafficDataManager  *DataManager;
    TRouter              *Router;
    TSynchronize         *Sync;
    TRandom              *Random;
    TString               RouteplanFilename;
    TStream              *AgeStatOutput;
    TNodeList             Nodes;
    TEdgeList             Edges;
    double                Sample;
    long                  MaxRoutes;
    long                  MinTime, MaxTime;
    long                  MinTimeFound, MaxTimeFound;
    TStream              *LinkHitStatOutput;
    long                  LinkHitBinSize;
    long                  RoutesClipped;
    long                  RoutesRead;
    long                  RoutesSampled;
    int                   DepartureBinSize;
    int                   NrOfDepartureBins;
    long                 *DepartureBins;
    long                 *ReroutingTimeCounts;
    bool                  Binary;

    bool
    Activate(void);

    bool
    ActivateAgeStat(void);

    bool
    ActivateLinkHitStat(void);

    bool
    Clip(void);

    bool
    ClipRoute(TParser *Parser);

    bool
    DetermineNetBoundaries(void);

    TString
    GetClassName(void) const;

    TString
    GetInstanceName(void) const;

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);

    virtual bool
    HandleLoadRequest(TParallelObject *Object, bool &Loaded);

    bool
    Hit(TParallelID EdgeID, long TimeStep);

    bool
    ReadNetwork(void);

    bool
    SortRouteplans(void) const;

    bool
    WriteRoute(const TTimeStep &TimeStep, TVehicle *Vehicle);
    
    bool
    WriteAgeStat(void);

    bool
    WriteLinkHitStat(void);

    bool
    WriteStatistics(void);
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
