/*      file     : TrafficData/TAccessory.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TAccessory.h,v 1.5 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TACCESSORY_H
#define TACCESSORY_H

// ***INCLUDES************************************************************************ 

#include <TParallelListObject.h>
#include <TTimeStep.h>
#include <TDLList.h>

// ***FORWARDS************************************************************************

#include <TEdge.forward.h>

class TAccessory;
class TBaseStream;
class TDistributedRange;
class TSource;
class TSink;
class TVehicle;

// ***CLASSES************************************************************************* 

class TAccessoryConstData

{
public:

    TAccessoryConstData(void);

    TParallelID   EdgeID;
    TParallelID   DestNodeID;
    float         RelativeLocation;
    float         AbsoluteLocation;
};

// ===================================================================================

class TAccessory : public TParallelListObject

{
public:

    TAccessory(TParallelID theParallelID, TAccessoryConstData *ConstData = 0);
    ~TAccessory(void);

    bool
    Activate(void);

    bool
    AddVehicle(TVehicle *Vehicle);

    void
    ComputeRelativeLocation(TEdge *Edge, TInfoID AdjacenceInfoID);

    void
    Dispose(void);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    inline float
    GetAbsoluteLocation(void) const;

    inline long
    GetDestNodeID(void) const;

    inline long
    GetEdgeID(void) const;

    virtual TString
    GetInstanceName(void) const;

    static TAccessory *
    GetPtr(TParallelID ID);

    inline float
    GetRelativeLocation(void) const;

    virtual bool
    IsInRange(const TDistributedRange *Range) const;

    static TClassInfo *ClassInfo;

private:

    TSource                 *Source;
    TSink                   *Sink;
    TAccessoryConstData      Const;

    // temporary data:
    
    long                     Offset;    
    
    bool
    AbsorbVehicles(TTimeStep TimeStep);

    virtual TString
    GetClassName(void) const;

    bool
    InsertVehicles(TTimeStep TimeStep);

    void
    _Init(void);

};

// ***TYPES*************************************************************************** 

typedef TDLList< TAccessory > TAccessoryList;
typedef TDLListEnum< TAccessory > TAccessoryEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TAccessory.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
