/*      file     : TrafficData/TIntersectionControl.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TIntersectionControl.h,v 1.5 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TINTERSECTIONCONTROL_H
#define TINTERSECTIONCONTROL_H

// ***INCLUDES************************************************************************

#include <TParallelObject.h>

// ***FORWARDS************************************************************************

class TAsciiDataTypeReader;
class TBaseStream;
class TIncidenceInfo;
class TParser;
class TPhase;
class TRandom;
class TTimeStep;
class TTransferInfo;

// ***DEFINES************************************************************************* 
 
#define IC_AVERAGE_TRANSFER_ACTIVE     1
#define IC_AVERAGE_TRANSFER_CORRECTION 2

// ***CLASSES*************************************************************************

class TIntersectionControl : public TParallelObject

{
public:
    
    TIntersectionControl(TParallelID theParallelID);
    virtual ~TIntersectionControl(void);

    bool
    Activate(TIncidenceInfo *theIncidenceInfo);

    bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    inline double
    GetGreenRatio(int EdgeNr) const;

    inline int
    GetNrOfPhases(void) const;

    inline bool
    IsIncomingSegmentActive(int EdgeNr);

    inline bool
    IsIncomingVehicleActive(int EdgeNr);

    bool
    LoadPhases(TAsciiDataTypeReader *Reader);

    static TClassInfo *ClassInfo;
    
private:

    // const data

    short int               NrOfPhases;
    short int               Flags;

    // var data

    TPhase                **Phases;

    // temp data

    TRandom                *Random;
    TIncidenceInfo         *IncidenceInfo;

    bool
    Allocate(void);

    void
    ComputeOffsets(void);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual TString
    GetInstanceName(void) const;

    void
    _Init(void);

    bool
    SortPhases(void);

    void
    UpdateActiveFlags(void);
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TIntersectionControl.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
