/*      file     : TrafficData/TRoutingStat.i.C
        project  : PAMINA
        authors  : Marcus Rickert
        RCS      : $Id: TRoutingStat.i.C,v 1.3 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TSource.h>
#include <TSink.h>

// ***METHODS************************************************************************* 

inline void
TRoutingStat::AddVehicles(long theNrOfVehicles)

{
    NrOfVehicles += theNrOfVehicles;
}

// =================================================================================== 

inline void
TRoutingStat::Collect(const TSource *Source)

{
    NormalInsertions += Source->Var.NormalInsertions;
    DelayedInsertions += Source->Var.DelayedInsertions;
    PendingVehicles += Source->Var.PendingVehicles->GetNrOfElements();
    SumDelay += Source->Var.SumDelay;
    NrOfSources++;
}

// =================================================================================== 

inline void
TRoutingStat::Collect(const TSink *Sink)

{
    Deletions += Sink->Var.Deletions;
    RoutesFailed += Sink->Var.RoutesFailed;
    NrOfSinks++;
}

// =================================================================================== 

inline void
TRoutingStat::CountRouteplanFile(void)

{
    OpenRouteplanFiles++;
}

// =================================================================================== 

inline long
TRoutingStat::GetDeletions(void) const

{
    return Deletions;
}

// =================================================================================== 

inline long
TRoutingStat::GetDelayedInsertions(void) const

{
    return DelayedInsertions;
}

// =================================================================================== 

inline long
TRoutingStat::GetInsertions(void) const

{
    return DelayedInsertions + NormalInsertions;
}

// =================================================================================== 

inline long
TRoutingStat::GetNormalInsertions(void) const

{
    return NormalInsertions;
}

// =================================================================================== 

inline long
TRoutingStat::GetNrOfRoutedVehicles(void) const

{
    return NormalInsertions + DelayedInsertions - Deletions;
}

// =================================================================================== 

inline long
TRoutingStat::GetNrOfSources(void) const

{
    return NrOfSources;
}

// =================================================================================== 

inline long
TRoutingStat::GetNrOfSinks(void) const

{
    return NrOfSinks;
}

// =================================================================================== 

inline long
TRoutingStat::GetNrOfVehicles(void) const

{
    return NrOfVehicles;
}

// =================================================================================== 

inline long
TRoutingStat::GetOpenRouteplanFiles(void) const

{
    return OpenRouteplanFiles;
}

// =================================================================================== 

inline long
TRoutingStat::GetPendingVehicles(void) const

{
    return PendingVehicles;
}

// =================================================================================== 

inline double
TRoutingStat::GetRatioOfRoutedVehicles(void) const

{
    if (NrOfVehicles == 0)
        return -1.0;
    else
        return 1.0 * GetNrOfRoutedVehicles() / NrOfVehicles;
}

// =================================================================================== 

inline long
TRoutingStat::GetRoutesFailed(void) const

{
    return RoutesFailed;
}

// =================================================================================== 
