/*      file     : TrafficData/TVehicle.h
        project  : Parallel Traffic Microsimulation (ZPR)
        author   : Marcus Rickert
        RCS      : $Id: TVehicle.h,v 1.6 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TVEHICLE_H
#define TVEHICLE_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TParallelListObject.h>

#include "CAConfig.h"

// ***FORWARDS************************************************************************

class TBaseStream;
class TClassInfo;
class TParser;
class TRouteplan;
class TTransferInfo;

// ***DEFINES************************************************************************* 

#define V_FLAG_TRACE            PO_FLAG_FIRST_FREE_FLAG
#define V_FLAG_REFLECT         (V_FLAG_TRACE << 1)
#define V_FLAG_ROUTEPLAN       (V_FLAG_TRACE << 2)
#define V_FLAG_FIRST_FREE_FLAG (V_FLAG_TRACE << 3)

// ***CLASSES************************************************************************* 

class TVehicle : public TParallelListObject

{
 public:
    
    TVehicle(TRouteplan *theRouteplan, TBaseStream *Stream, TTransferInfo *TransferInfo);
    TVehicle(TParallelID theParallelID);
    virtual ~TVehicle(void);

    inline void
    ActivateTrace(void);

    virtual TBaseObject *
    Copy(void) const;

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *) const;

    virtual TSite
    GetCAVelocity(void) const;

    virtual TString
    GetInstanceName(void) const;

    inline long
    GetLane(void) const;

    virtual TSite
    GetMaxCAVelocity(void) const;

    inline double
    GetMaxVelocity(void) const;

    inline long
    GetOffset(void) const;

    inline TRouteplan *
    GetRouteplan(void);

    virtual TSite
    GetSitePattern(void) const;

    inline void
    SetCurrentVelocity(double theCurrentVelocity);

    inline void
    SetCurrentVelocityInCAUnits(long theCurrentVelocity);

    inline void
    SetLane(long theLane);

    inline void
    SetMaxVelocityInCAUnits(long theMaxVelocity);

    inline void
    SetMaxVelocity(double theMaxVelocity);
    
    inline void
    SetOffset(long theOffset);

    inline void
    SetRouteplan(TRouteplan *theRouteplan);
    
    virtual void
    SetSitePattern(TSite Site);

    inline bool
    TraceActive(void) const;

    static TClassInfo          *ClassInfo;
    static TDLList< TVehicle >  UnusedVehicles;

 protected:
    
    virtual TString
    GetClassName(void) const;

 private:

    TRouteplan *Routeplan;

    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define VEHICLE_FLOAT 2
    float             CurrentVelocity;
    float             MaxVelocity;
#   define VEHICLE_LONG 2
    long              Lane;
    long              Offset;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

    // temporary data used for boundary handling


    void
        _Init(void);
};

// ***TYPES*************************************************************************** 

typedef TDLList<TVehicle> TVehicleList;
typedef TDLListEnum<TVehicle> TVehicleEnum;

// ***INLINE METHODS****************************************************************** 

#ifndef NO_INLINE
#  include "TVehicle.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
