/*      file     : XTrafficData/TMapView.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TMapView.h,v 1.4 2005/03/25 12:29:05 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMAPVIEW_H
#define TMAPVIEW_H

// ***INCLUDES************************************************************************ 

#include <TLocation.h>
#include <TWindow.h>
#include <TStatHandler.h>
#include <TDLList.h>
#include <TTimeStep.h>

// ***FORWARDS************************************************************************

class TAggregationInfo;
class TEdgeInfo;
class TGraphEdgeInfo;
class TGridStatLookup;
class TMapViewer;
class TSamplingDescription;
class TTimeStep;

#include <TEdge.forward.h>

// ***TYPES***************************************************************************

enum TViewMode

{
    LineViewMode,
    VelocityViewMode,
    DensityViewMode,
    DumpViewMode,
    DelayViewMode,
};

// ***CLASSES************************************************************************* 

class TMapView : public TWindow, public TDLNodePrimitive, public TStatHandler

{
public:

    TMapView(TMapViewer *Viewer, const char *Name, TRectangle *aFrame, 
             TEdgeList *theEdges, int theWidth, int theHeight);
    virtual ~TMapView(void);

    virtual TString
    GetInstanceName(void) const;

    const TRectangle &
    GetFrame(void) const;

private:

    bool
    ModifyFrame(double PercentX, double PercentY, double Zoom);

    bool
    CheckVisibleEdges(void);

    virtual bool
    Draw(void);
    
    void
    DrawColoredEdge(TGraphEdgeInfo *GraphEdge, int Color);

    bool
    DrawDensity(TEdge *Edge, TGraphEdgeInfo *GraphEdge);

    bool
    DrawDump(TEdge *Edge, TGraphEdgeInfo *Info);

    bool
    DrawLine(TEdge *Edge, TGraphEdgeInfo *GraphEdge);

    bool
    DrawMap(void);

    bool
    DrawMisc(void);
    
    bool
    DrawVelocity(TEdge *Edge, TGraphEdgeInfo *GraphEdge);

    virtual TString
    GetClassName(void) const;

    bool
    GetGraphicData(TEdge *Edge, TEdgeInfo *&EdgeInfo, TGridStatLookup *&GridStatLookup);

    virtual bool
    HandleButtonPress(int x, int y, int ButtonState);

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    bool
    HandleDumpResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    virtual bool
    HandleKeyPress(int KeyCode);

    virtual void
    LabelDump(const char *Label);

    bool
    SelectMode(TViewMode NewMode);

    bool
    StartMultilaneDump(void);

    void
    StopMultilaneDump(void);

    bool         MarkActive;
    TLocation    FirstMark;
    TLocation    SecondMark;
    TViewMode    Mode;
    TMapViewer  *MapViewer;
    TEdgeList   *Edges;
    TEdgeList   *VisibleEdges;
    TRectangle   Frame;
    bool         FrameChanged;
    TTimeStep    DisplayTime;
    int          ColorBase;

    TSamplingDescription *DumpDesc;
    TAggregationInfo     *DumpInfo;

    double       VehicleRadius;
    int          BackgroundColor;
    int          VehicleColor;
    int          BlockColor;
    int          AbsorptionColor;
    int          EmissionColor;
    int          BoxLineColor;
    int          LineColor;    
    int          Font;
    int          DumpInterval;
};

// ***TYPES*************************************************************************** 

typedef TDLList< TMapView > TMapViewList;
typedef TDLListEnum< TMapView > TMapViewEnum;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
