/*      file     : common/TABTree.h
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TABTreeElement
                   TABTreePrimitive
                   TABTree<>
        RCS      : $Id: TABTree.h,v 1.9 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifdef __GNUC__
#pragma interface
#endif

#ifndef TABTREE_H
#define TABTREE_H

// ***INCLUDES************************************************************************

#include <Basic.h>
#include <TContainer.h>
#include <TEnumerator.h>
#include <TSelector.h>

// ***FORWARDS************************************************************************

class TABTreeElement;

// ***CLASSES*************************************************************************

/**
  */

class TABTreePrimitive : public TContainer

{
public:
    
    TABTreePrimitive(long theFlags = CM_DEFAULT_MODE);
    virtual ~TABTreePrimitive(void);

    bool
        Apply(TApplyFunction Function, void *Context = 0);
    bool
        Check(void);
    TBaseObject *
        DeleteByValue(TBaseObject *Item, int CompareCode);
    virtual TBaseObject *
        _DeleteByReference(TBaseObject *Object, int CompareCode);
    void            
        DeleteAll(void);
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;
    virtual TBaseObject *
        _Find(TBaseObject *Element, int CompareCode);
    virtual long
        GetNrOfElements(void) const;
    TABTreeElement *
        GetRoot(void) const;
    virtual bool        
        _Insert(TBaseObject *Element, bool Unique, int CompareCode);
    virtual void
        MoveBackward(TEnumeratorPrimitive *Enumerator);
    virtual void
        MoveForward(TEnumeratorPrimitive *Enumerator);
    virtual bool
        WhileEnumeration(TEnumeratorPrimitive *Enumerator);
    virtual bool
        WhileSelection(TSelectorPrimitive *Selector);

protected:
    
    void
        Init(void);
    
private:

    int             NrOfElements;
    TABTreeElement *Root;
    
    TBaseObject *
        _Delete(TABTreeElement *Element, int DeletionPoint);
    virtual void
        Enumerate(TEnumeratorPrimitive *Enumerator);
    int
        FindItem(TABTreeElement *Node, TBaseObject *Item, bool& Found, 
                 int CompareCode, bool CheckReference);
    TABTreeElement *
        FindLeaf(TBaseObject *Element, int CompareCode, bool CheckReference);
    TABTreeElement *
        FindLeftMostLeaf(void);
    virtual TBaseObject *
        GetCurrent(TEnumeratorPrimitive *Enumerator);
    virtual void
        Select(TSelectorPrimitive *Selector);
};

// ===================================================================================

/**
  */

template <class EntryType>
class TABTree : public TABTreePrimitive

{
 public:
    
    TABTree(long theFlags = CM_DEFAULT_MODE);

    bool
        AddCopies(TABTree <EntryType > *aContainer, bool Unique = TRUE, int CompareCode = 0);
    EntryType *
        Delete(EntryType Entry, int CompareCode = 0);
    EntryType *
        DeleteByReference(EntryType *Entry, int CompareCode = 0);
    EntryType *
        DeleteByReference(EntryType Entry, int CompareCode = 0);
    bool
        Contains(EntryType Entry, int CompareCode = 0);
    EntryType *
        Find(EntryType Entry, int CompareCode = 0);
    bool
        Insert(EntryType *Entry, bool Unique, int CompareCode = 0);
};

// ***TEMPLATE METHODS****************************************************************

#if defined (INCLUDE_TEMPLATES) || defined(TEMPLATEBODIES)
#include "TABTree.t.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
// =================================================================================== 
#endif

