/*      file     : common/TArray.h
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TArrayPrimitive
                   TArray<>
        RCS      : $Id: TArray.h,v 1.13 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifdef __GNUC__
#pragma interface
#endif

#ifndef TARRAY_H
#define TARRAY_H

// ***INCLUDES************************************************************************

#include <TContainer.h>
#include <TDebug.h>
#include <TString.h>

// ***DEFINES*************************************************************************

#define TARRAY_DEFAULT_INCREMENT 8

// ***CLASSES*************************************************************************

/** Linear array.
  */

class TArrayPrimitive : public TContainer

{
  public:
    
    bool
        Allocate(long theNrOfItems);
    void
        Delete(long Index);
    virtual TString
        GetInstanceName(void) const;
    inline long
        GetNrOfItems(void) const;
    inline long
        GetPhysicalNrOfItems(void) const;
    bool
        _InsertAt(long Index, void *Data);
    bool
        Resize(long aNrOfItems);

  protected:

    short int  DeltaItems;
    long       LogicalNrOfItems;
#ifdef DARRAY_VAR_ITEMSIZE
    short int  ItemSize;
#endif
    char      *Items;
#ifdef DARRAYNAME
    TString Name;
#endif
    
#ifdef DARRAY_VAR_ITEMSIZE
    TArrayPrimitive(int theItemSize, int theIncrement, char *theName ="");
#else
    TArrayPrimitive(int theIncrement, char *theName ="");
#endif
    virtual ~TArrayPrimitive(void);
    
    virtual bool
        ActionBeforeDeletion(long First, long Last);
    virtual bool
        ActionAfterInsertion(long First, long Last);
    bool
        _Add(void *Item);
#ifdef DARRAY_VAR_ITEMSIZE
    inline long
        GetItemSize(void) const;
#else
    virtual long
        GetItemSize(void) const;
#endif

  private:
    
    short int Increment;
    
    void
        Deallocate(void);
    bool
        Enlarge(void);
    void
        _Set(long Index, void *Data);
    void
        SetLogicalNrOfItems(long theNumber);    
};

// ===================================================================================

/** Template for linear arrays.
  */

template <class T>
class TArray : public TArrayPrimitive

{
  public:

    TArray(char *theName);
    TArray(int theIncrement = TARRAY_DEFAULT_INCREMENT, char *theName = "");

  
    inline T &
        operator [] (long Index);    
    inline const T &
        operator [] (long Index) const;
    bool
        operator = (const TArray<T> &anArray);
    bool
        operator += (const TArray<T> &anArray);

    bool
        Add(const T &Item);
    T &
        Get(long Index);
    const T &
        Get(long Index) const;
    virtual TString
        GetClassName(void) const;
    T *
        GetItemPtr(void);
    const T *
        GetItemPtr(void) const;
#ifndef DARRAY_VAR_ITEMSIZE
    virtual long
        GetItemSize(void) const;
#endif
    bool
        InsertAt(long Index, T &Item);
    void
        Set(long Index, T& Item);
};

// ***TEMPLATE METHODS****************************************************************

#if defined (INCLUDE_TEMPLATES) || defined(TEMPLATEBODIES)
#include "TArray.t.C"
#endif

// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#include "TArray.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
