/*      file     : common/TContainer.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TContainer.h,v 1.4 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCONTAINER_H
#define TCONTAINER_H

// ***INCLUDES************************************************************************ 

#include "TBaseObject.h"

// ***FORWARDS************************************************************************

class TEnumeratorPrimitive;
class TSelectorPrimitive;

// ***DEFINES************************************************************************* 

#define CM_AUTO_DISPOSE_ENTRIES BO_FLAG_FIRST_FREE_FLAG
#define CM_AUTO_DISPOSE_OBJECTS (CM_AUTO_DISPOSE_ENTRIES << 1)
#define CM_WARN_IF_NOT_EMPTY    (CM_AUTO_DISPOSE_OBJECTS << 1)
#define CM_DUMP_ENTRIES         (CM_WARN_IF_NOT_EMPTY << 1)
#define CM_AUTO_DISPOSE (CM_AUTO_DISPOSE_ENTRIES | CM_AUTO_DISPOSE_OBJECTS)
#define CM_DEFAULT_MODE          CM_AUTO_DISPOSE_ENTRIES
#define CM_VERBOSE_MODE (CM_DEFAULT_MODE | CM_WARN_IF_NOT_EMPTY | CM_DUMP_ENTRIES)

// ***CLASSES************************************************************************* 

/** Base class to all container classes.
  */

class TContainer : public TBaseObject

{
public:

    TContainer(long theFlags = CM_DEFAULT_MODE);

    operator TContainer * (void);

    inline bool
        AutoDisposeEntries(void) const;
    inline bool
        AutoDisposeObjects(void) const;
    virtual bool
        _AddCopies(TContainer *aContainer, int CompareCode = 0, bool Unique = TRUE);
    bool
        CheckSort(int CompareCode = 0);
    int 
        CompareItems(TBaseObject *Key, TBaseObject *Local, int CompareCode, 
                     bool CheckReference);
    virtual bool
        _Contains(TBaseObject *Entry, int CompareCode = 0);
    virtual TBaseObject *
        _DeleteByReference(TBaseObject *Entry, int CompareCode = 0);
    inline TBaseObject *
        _DisposeContainerEntry(TBaseObject *Entry);
    virtual void
        DumpEntries(void);
    inline bool
        DumpEntriesOnDispose(void) const;
    virtual void
        Enumerate(TEnumeratorPrimitive *Enumerator);
    virtual TBaseObject *
        _Find(TBaseObject *Entry, int CompareCode = 0);
    virtual TBaseObject *
        GetCurrent(TEnumeratorPrimitive *Enumerator);
    virtual long
        GetNrOfElements(void) const;
    virtual bool
        _Insert(TBaseObject *Entry, bool Unique, int CompareCode = 0);
    virtual void
        MoveBackward(TEnumeratorPrimitive *Enumerator);
    virtual void
        MoveForward(TEnumeratorPrimitive *Enumerator);
    virtual void
        Select(TSelectorPrimitive *Selector);
    inline bool
        WarnAboutEntriesOnDispose(void) const;
    virtual bool
        WhileEnumeration(TEnumeratorPrimitive *Enumerator);
    virtual bool
        WhileSelection(TSelectorPrimitive *Selector);

};

// ***PROTOTYPES********************************************************************** 
#ifdef __cplusplus
extern "C" {
#endif
    TBaseObject *
        DisposeContainerEntry(TBaseObject *Entry, bool DisposeEntry = TRUE,
                              bool DisposeObject = FALSE);
#ifdef __cplusplus
}
#endif
// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#  include "TContainer.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
