/*      file     : common/TDebug.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TDebug.h,v 1.15 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TDEBUG_H
#define TDEBUG_H

// ***INCLUDES************************************************************************

#include <stdarg.h>

#include <Basic.h>
#include <TString.h>

// ***DEFINES*************************************************************************

#ifdef TDEBUG
#define debug(Command) Command
#define IfDebug(Mask, Command) { if (Debug.GetMask() & (Mask)) Command; }
#define trace(MethodName) TDebug __Debug(GetClassName(), MethodName, GetInstanceName()); \
        __Debug.AvoidWarning()
#else
#define debug(Command)
#define IfDebug(Mask, Command)
#define trace(MethodName)
#endif

#define DM_LIFESPAN       0x00000001
#define DM_CONTROL        0x00000002
#define DM_CODE           0x00000004
#define DM_EVENT          0x00000008
#define DM_MSG            0x00000010
#define DM_MSG_CONTROL    0x00000020
#define DM_PARSE          0x00000040
#define DM_PARALLEL       0x00000080
#define DM_STRING         0x00000100
#define DM_AB_TREE        0x00000200
#define DM_SIGNAL         0x00000400
#define DM_ALARM          0x00000800
#define DM_LEX            0x00001000
#define DM_YACC           0x00002000
#define DM_CONFIG         0x00004000
#define DM_CLASS          0x00008000
#define DM_BOUNDARY       0x00010000
#define DM_STAT_SAMPLE    0x00020000
#define DM_STAT_AGGR      0x00040000
#define DM_TIME_STEP      0x00080000
#define DM_MIGRATION      0x00100000
#define DM_TIME           0x00200000
#define DM_LOAD           0x00400000
#define DM_CONTAINER      0x00800000
#define DM_OBJECT         0x01000000
#define DM_FILE           0x02000000
#define DM_BALANCE        0x04000000
#define DM_COLLISION      0x08000000
#define DM_TOPOLOGY       0x10000000
#define DM_ROUTE          0x20000000

// ***CLASSES*************************************************************************

/**
  */

class TDebug

{
 public:

    TDebug(const char *ClassNamePar, const char *MethodNamePar, const char *InstanceName);
    ~TDebug(void);

    void
        AvoidWarning(void);

    static void
        Activate(void);
    static void
        Deactivate(void);
    static unsigned long
        GetMask(void);
    void
        Message(const char *AMessage ...);
    static void
        SetMask(unsigned long theMask);
    
  private:

    static int Indent;
    static bool Active;
    static unsigned long Mask;
    
    TString MethodName;
    TString ClassName;
};

// ***EXTERNALS***********************************************************************

extern TDebug Debug;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
