/*      file     : common/TStream.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TStream.h,v 1.7 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSTREAM_H
#define TSTREAM_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <Basic.h>
#include <TString.h>
#include <TBaseStream.h>

// ***DEFINES*************************************************************************

#define MAX_INPUT_LINE_LENGTH    16384
#define COMMENT_CHAR             '#'
#define QUOTATION_CHAR           '"'
#define QUOTATION_CHAR_DELIMITER """"
#define TAB_CHAR                 '\t'
#define NEWLINE_CHAR             '\n'
#define DEFAULT_DELIMITERS       " "

// ***CLASSES*************************************************************************

/**
  */

class TStream : public TBaseStream

{
 public:

    TStream(char *theDelimiters = " ", 
            long MaxInputLineLength = MAX_INPUT_LINE_LENGTH);
    virtual ~TStream(void);

#include "TBaseStream.Headers.h"

    inline 
    operator FILE * (void);

    void
    ClearError(void);

    void
    Close(void);

    // this declaration should be generated and located in TBaseStream.Headers.h
    virtual bool
    Decode(LongID &Value, long Count = 1, long = 1);

    // this declaration should be generated and located in TBaseStream.Headers.h
    virtual bool
    Encode(const LongID &Value, long Count = 1, long = 1) const;

    bool
    EndOfFile(void) const;

    void
    Flush(void);

    FILE *
    GetFile(void);

    virtual TString
    GetInstanceName(void) const;

    inline long
    GetLineNr(void) const;

    long
    GetPercentage(void);

    int
    GetTokenID(long ExpectedToken = -1) const;

    virtual bool
    IsBinary(void) const;

    bool
    Open(const char *Filename, const char *Mode = "r", bool BinaryFlag = FALSE);

    bool
    Read(short int &Value);

    bool
    Read(unsigned short int &Value);

    bool
    Read(long &Value);

    bool
    Read(double &Value);

    bool
    Read(float &Value);

    bool
    Read(char *Buffer);

    bool
    Read(LongID &Value);

    const char *
    ReadToken(void);

    virtual bool
    Write(const char *Format, ...);

protected:

    long      MaxInputLineLength;
    char     *InputLine;
    long      LineNr;
    long      LinePos;
    char     *CurrentPosition;
    char     *CurrentDelimiters;
    char     *Delimiters;
    char     *CurrentToken;
    bool      Binary;

    virtual TString
    GetClassName(void) const;

    bool
    ReadLine(void);

private:

    TString   Filename; 
    FILE     *File;
    bool      IsPipe;

    inline void
    SkipWhiteChars(void);
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TStream.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
