/*      file     : graphx/Graph.h
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: Graphx.h,v 1.6 2005/03/25 11:06:10 mr Exp $
*/

#ifndef __Graphx__
#define __Graphx__

/********************************************************************/
/* Plans for version 2.0:                                           */
/* 1) Support for fontdrawing.                                      */
/*                                                                  */
/* Even further in the future, there might be support for 3D stuff -*/
/* Who knows ???                                                    */
/********************************************************************/

#include <graphx.h>

#ifdef DEBUG

#ifdef VERSION
#undef VERSION
#endif

#define VERSION 1.99

#ifdef PACKAGE
#undef PACKAGE
#endif

#define PACKAGE "Graphx (C++)"

#endif

typedef void VOID;
typedef char CHAR;
typedef unsigned char U_CHAR;
typedef short SHORT;
typedef unsigned short U_SHORT;
typedef int INT;
typedef unsigned UNSIGNED;
typedef long LONG;
typedef unsigned long U_LONG;

typedef float FLOAT;
typedef double REAL;
typedef long double L_REAL;

struct GrPoint {
  INT color;
  REAL x,y;
  GrPoint() {
  #ifdef DEBUG
    SETFUNC("GrPoint::GrPoint()");
  #endif
    x = y = color = 0;};
  GrPoint(REAL xx,REAL yy,INT c) {
  #ifdef DEBUG
    SETFUNC("GrPoint::GrPoint(REAL,REAL,INT)");
  #endif
    x=xx;y=yy;color=c;
  };
  ~GrPoint() {
  #ifdef DEBUG
    SETFUNC("GrPoint::~GrPoint()");
  #endif
  };
};

struct GrPointPS {
  INT color;
  REAL x,y;
  GrPointPS() {
  #ifdef DEBUG
    SETFUNC("GrPointPS::GrPointPS()");
  #endif
    x = y = color = 0;
  };
  GrPointPS(REAL xx,REAL yy,INT c) {
  #ifdef DEBUG
    SETFUNC("GrPointPS::GrPointPS(REAL,REAL,INT)");
  #endif
    x=xx;y=yy;color=c;
  };
  ~GrPointPS() {
  #ifdef DEBUG
    SETFUNC("GrPointPS::~GrPointPS()");
  #endif
  };
};

struct GrLine {
  INT color;
  REAL x1,y1,x2,y2;
  GrLine() {
  #ifdef DEBUG
    SETFUNC("GrLine::GrLine()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrLine(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrLine::GrLine(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrLine() {
  #ifdef DEBUG
    SETFUNC("GrLine::~GrLine()");
  #endif
  };
};

struct GrLinePS {
  INT color;
  REAL x1,y1,x2,y2;
  GrLinePS() {
  #ifdef DEBUG
    SETFUNC("GrLinePS::GrLinePS()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrLinePS(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrLinePS::GrLinePS(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrLinePS() {
  #ifdef DEBUG
    SETFUNC("GrLinePS::~GrLinePS()");
  #endif
  };
};

struct GrBox {
  INT color;
  REAL x1,y1,x2,y2;
  GrBox() {
  #ifdef DEBUG
    SETFUNC("GrBox::GrBox()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrBox(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrBox::GrBox(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrBox() {
  #ifdef DEBUG
    SETFUNC("GrBox::~GrBox()");
  #endif
  };
};

struct GrBoxPS {
  INT color;
  REAL x1,y1,x2,y2;
  GrBoxPS() {
  #ifdef DEBUG
    SETFUNC("GrBoxPS::GrBoxPS()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrBoxPS(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrBoxPS::GrBoxPS(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrBoxPS() {
  #ifdef DEBUG
    SETFUNC("GrBoxPS::~GrBoxPS()");
  #endif
  };
};

struct GrFillBox {
  INT color;
  REAL x1,y1,x2,y2;
  GrFillBox() {
  #ifdef DEBUG
    SETFUNC("GrFillBox::GrFillBox()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrFillBox(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillBox::GrFillBox(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrFillBox() {
  #ifdef DEBUG
    SETFUNC("GrFillBox::~GrFillBox()");
  #endif
  };
};

struct GrFillBoxPS {
  INT color;
  REAL x1,y1,x2,y2;
  GrFillBoxPS() {
  #ifdef DEBUG
    SETFUNC("GrFillBoxPS::GrFillBoxPS()");
  #endif
    x1 = y1 = x2 = y2 = color = 0;
  };
  GrFillBoxPS(REAL xx1,REAL yy1,REAL xx2,REAL yy2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillBoxPS::GrFillBoxPS(REAL,REAL,REAL,REAL,INT)");
  #endif
    x1 = xx1;y1 = yy1;x2 = xx2;y2 = yy2;color = c;
  };
  ~GrFillBoxPS() {
  #ifdef DEBUG
    SETFUNC("GrFillBoxPS::~GrFillBoxPS()");
  #endif
  };
};

struct GrEllipticArc {
  INT color;
  REAL x,y,a,b,a1,a2;
  GrEllipticArc() {
  #ifdef DEBUG
    SETFUNC("GrEllipticArc::GrEllipticArc()");
  #endif
    x = y = a = b = a1 = a2 = color = 0;
  };
  GrEllipticArc(REAL xx,REAL yy,REAL aa,REAL bb,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrEllipticArc::GrEllipticArc(REAL,REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrEllipticArc() {
  #ifdef DEBUG
    SETFUNC("GrEllipticArc::~GrEllipticArc()");
  #endif
  };
};

struct GrEllipticArcPS {
  INT color;
  REAL x,y,a,b,a1,a2;
  GrEllipticArcPS() {
  #ifdef DEBUG
    SETFUNC("GrEllipticArcPS::GrEllipticArcPS()");
  #endif
    x = y = a = b = a1 = a2 = color = 0;
  };
  GrEllipticArcPS(REAL xx,REAL yy,REAL aa,REAL bb,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrEllipticArcPS::GrEllipticArcPS(REAL,REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrEllipticArcPS() {
  #ifdef DEBUG
    SETFUNC("GrEllipticArcPS::~GrEllipticArcPS()");
  #endif
  };
};

struct GrFillEllipticArc {
  INT color;
  REAL x,y,a,b,a1,a2;
  GrFillEllipticArc() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArc::GrFillEllipticArc()");
  #endif
  x = y = a = b = a1 = a2 = color = 0;
  };
  GrFillEllipticArc(REAL xx,REAL yy,REAL aa,REAL bb,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArc::GrFillEllipticArc(REAL,REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrFillEllipticArc() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArc::~GrFillEllipticArc()");
  #endif
  };
};

struct GrFillEllipticArcPS {
  INT color;
  REAL x,y,a,b,a1,a2;
  GrFillEllipticArcPS() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArcPS::GrFillEllipticArcPS()");
  #endif
    x = y = a = b = a1 = a2 = color = 0;
  };
  GrFillEllipticArcPS(REAL xx,REAL yy,REAL aa,REAL bb,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArcPS::GrFillEllipticArcPS(REAL,REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrFillEllipticArcPS() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipticArcPS::~GrFillEllipticArcPS()");
  #endif
  };
};

struct GrEllipse {
  INT color;
  REAL x,y,a,b;
  GrEllipse() {
  #ifdef DEBUG
    SETFUNC("GrEllipse::GrEllipse()");
  #endif
    x = y = a = b = color = 0;};
  GrEllipse(REAL xx,REAL yy,REAL aa,REAL bb,INT c) {
  #ifdef DEBUG
    SETFUNC("GrEllipse::GrEllipse(REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;color = c;
  };
  ~GrEllipse() {
  #ifdef DEBUG
    SETFUNC("GrEllipse::~GrEllipse()");
  #endif
  };
};

struct GrEllipsePS {
  INT color;
  REAL x,y,a,b;
  GrEllipsePS() {
  #ifdef DEBUG
    SETFUNC("GrEllipsePS::GrEllipsePS()");
  #endif
    x = y = a = b = color = 0;};
  GrEllipsePS(REAL xx,REAL yy,REAL aa,REAL bb,INT c) {
  #ifdef DEBUG
    SETFUNC("GrEllipsePS::GrEllipsePS(REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;color = c;
  };
  ~GrEllipsePS() {
  #ifdef DEBUG
    SETFUNC("GrEllipsePS::~GrEllipsePS()");
  #endif
  };
};

struct GrFillEllipse {
  INT color;
  REAL x,y,a,b;
  GrFillEllipse() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipse::GrFillEllipse()");
  #endif
    x = y = a = b = color = 0;};
  GrFillEllipse(REAL xx,REAL yy,REAL aa,REAL bb,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillEllipse::GrFillEllipse(REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;color = c;
  };
  ~GrFillEllipse() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipse::~GrFillEllipse()");
  #endif
  };
};

struct GrFillEllipsePS {
  INT color;
  REAL x,y,a,b;
  GrFillEllipsePS() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipsePS::GrFillEllipsePS()");
  #endif
    x = y = a = b = color = 0;};
  GrFillEllipsePS(REAL xx,REAL yy,REAL aa,REAL bb,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillEllipsePS::GrFillEllipsePS(REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;a = aa;b = bb;color = c;
  };
  ~GrFillEllipsePS() {
  #ifdef DEBUG
    SETFUNC("GrFillEllipsePS::~GrFillEllipsePS()");
  #endif
  };
};

struct GrArc {
  INT color;
  REAL x,y,r,a1,a2;
  GrArc() {
  #ifdef DEBUG
    SETFUNC("GrArc::GrArc()");
  #endif
    x = y = r = a1 = a2 = color = 0;
  };
  GrArc(REAL xx,REAL yy,REAL rr,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrArc::GrArc(REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrArc() {
  #ifdef DEBUG
    SETFUNC("~GrArc::GrArc()");
  #endif
  };
};

struct GrArcPS {
  INT color;
  REAL x,y,r,a1,a2;
  GrArcPS() {
  #ifdef DEBUG
    SETFUNC("GrArcPS::GrArcPS()");
  #endif
    x = y = r = a1 = a2 = color = 0;
  };
  GrArcPS(REAL xx,REAL yy,REAL rr,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrArcPS::GrArcPS(REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrArcPS() {
  #ifdef DEBUG
    SETFUNC("GrArcPS::~GrArcPS()");
  #endif
  };
};

struct GrFillArc {
  INT color;
  REAL x,y,r,a1,a2;
  GrFillArc() {
  #ifdef DEBUG
    SETFUNC("GrFillArc::GrFillArc()");
  #endif
    x = y = r = a1 = a2 = color = 0;
  };
  GrFillArc(REAL xx,REAL yy,REAL rr,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillArc::GrFillArc(REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrFillArc() {
  #ifdef DEBUG
    SETFUNC("GrFillArc::~GrFillArc()");
  #endif
  };
};

struct GrFillArcPS {
  INT color;
  REAL x,y,r,a1,a2;
  GrFillArcPS() {
  #ifdef DEBUG
    SETFUNC("GrFillArcPS::GrFillArcPS()");
  #endif
    x = y = r = a1 = a2 = color = 0;
  };
  GrFillArcPS(REAL xx,REAL yy,REAL rr,REAL aa1,REAL aa2,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillArcPS::GrFillArcPS(REAL,REAL,REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;a1 = aa1;a2 = aa2;color = c;
  };
  ~GrFillArcPS() {
  #ifdef DEBUG
    SETFUNC("GrFillArcPS::~GrFillArcPS()");
  #endif
  };
};

struct GrCircle {
  INT color;
  REAL x,y,r;
  GrCircle() {
  #ifdef DEBUG
    SETFUNC("GrCircle::GrCircle()");
  #endif
    x = y = r = color = 0;
  };
  GrCircle(REAL xx,REAL yy,REAL rr,INT c) {
  #ifdef DEBUG
    SETFUNC("GrCircle::GrCircle(REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;color = c;
  };
  ~GrCircle() {
  #ifdef DEBUG
    SETFUNC("GrCircle::~GrCircle()");
  #endif
  };
};

struct GrCirclePS {
  INT color;
  REAL x,y,r;
  GrCirclePS() {
  #ifdef DEBUG
    SETFUNC("GrCirclePS::GrCirclePS()");
  #endif
    x = y = r = color = 0;};
  GrCirclePS(REAL xx,REAL yy,REAL rr,INT c) {
  #ifdef DEBUG
    SETFUNC("GrCirclePS::GrCirclePS(REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;color = c;
  };
  ~GrCirclePS() {
  #ifdef DEBUG
    SETFUNC("GrCirclePS::~GrCirclePS()");
  #endif
  };
};

struct GrFillCircle {
  INT color;
  REAL x,y,r;
  GrFillCircle() {
  #ifdef DEBUG
    SETFUNC("GrFillCircle::GrFillCircle()");
  #endif
    x = y = r = color = 0;};
  GrFillCircle(REAL xx,REAL yy,REAL rr,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillCircle::GrFillCircle(REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;color = c;
  };
  ~GrFillCircle() {
  #ifdef DEBUG
    SETFUNC("GrFillCircle::~GrFillCircle()");
  #endif
  };
};

struct GrFillPolygon {
  INT color;
  INT shape;
  int mode;
  INT NrOfPoints;
  XPoint *Points;
  GrFillPolygon() 
  {
    Points = 0; color = shape = mode = NrOfPoints = 0;
  };
  GrFillPolygon(XPoint *thePoints, INT theNrOfPoints, INT theshape, INT themode, INT c) 
  {
    Points = thePoints; NrOfPoints = theNrOfPoints; shape = theshape; mode=themode; color = c;
  };
};

struct GrFillCirclePS {
  INT color;
  REAL x,y,r;
  GrFillCirclePS() {
  #ifdef DEBUG
    SETFUNC("GrFillCirclePS::GrFillCirclePS()");
  #endif
    x = y = r = color = 0;};
  GrFillCirclePS(REAL xx,REAL yy,REAL rr,INT c) {
  #ifdef DEBUG
    SETFUNC("GrFillCirclePS::GrFillCirclePS(REAL,REAL,REAL,INT)");
  #endif
    x = xx;y = yy;r = rr;color = c;
  };
  ~GrFillCirclePS() {
  #ifdef DEBUG
    SETFUNC("GrFillCirclePS::~GrFillCirclePS()");
  #endif
  };
};

struct GrChar {
  CHAR c;
  INT color;
  REAL x,y;
  GrChar() {
  #ifdef DEBUG
    SETFUNC("GrChar::GrChar()");
  #endif
    x = y = c = color = 0;};
  GrChar(REAL xx,REAL yy,CHAR cc,INT ccolor) {
  #ifdef DEBUG
    SETFUNC("GrChar::GrChar(REAL,REAL,CHAR,INT)");
  #endif
    x = xx;y = yy;c = cc;color = ccolor;
  };
  ~GrChar() {
  #ifdef DEBUG
    SETFUNC("GrChar::~GrChar()");
  #endif
  };
};

struct GrFillChar {
  CHAR c;
  INT fg,bg;
  REAL x,y;
  GrFillChar() {
  #ifdef DEBUG
    SETFUNC("GrFillChar::GrFillChar()");
  #endif
    x = y = c = fg = bg = 0;};
  GrFillChar(REAL xx,REAL yy,CHAR cc,INT ffg,INT bbg) {
  #ifdef DEBUG
    SETFUNC("GrFillChar::GrFillChar(REAL,REAL,CHAR,INT,INT)");
  #endif
    x = xx;y = yy;c = cc;fg = ffg;bg = bbg;
  };
  ~GrFillChar() {
  #ifdef DEBUG
    SETFUNC("GrFillChar::~GrFillChar()");
  #endif
  };
};

struct GrString {
  CHAR *s;
  INT color;
  REAL x,y;
  GrString() {
  #ifdef DEBUG
    SETFUNC("GrString::GrString()");
  #endif
    s = (CHAR *)NULL;x = y  = color = 0;};
  GrString(REAL xx,REAL yy,CHAR *ss,INT ccolor) {
  #ifdef DEBUG
    SETFUNC("GrString::GrString(REAL,REAL,CHAR,INT)");
  #endif
    x = xx;y = yy;s = ss;color = ccolor;
  };
  ~GrString() {
  #ifdef DEBUG
    SETFUNC("GrString::~GrString()");
  #endif
  };
};

struct GrFillString {
  CHAR *s;
  INT fg,bg;
  REAL x,y;
  GrFillString() {
  #ifdef DEBUG
    SETFUNC("GrFillString::GrFillString()");
  #endif
    s = (CHAR *)NULL;x = y  = fg = bg = 0;};
  GrFillString(REAL xx,REAL yy,CHAR *ss,INT ffg,INT bbg) {
  #ifdef DEBUG
    SETFUNC("GrFillString::GrFillString(REAL,REAL,CHAR,INT,INT)");
  #endif
    x = xx;y = yy;s = ss;fg = ffg;bg = bbg;
  };
  ~GrFillString() {
  #ifdef DEBUG
    SETFUNC("GrFillString::~GrFillString()");
  #endif
  };
};

struct GrWindow {
    grWindow *theWindow;
    
    GrWindow() {
    #ifdef DEBUG
      SETFUNC("GrWindow::GrWindow()");
    #endif
      theWindow = (grWindow *)NULL;
    };
    GrWindow(grWindow *theNewWindow) {
    #ifdef DEBUG
      SETFUNC("GrWindow::GrWindow(grWindow *)");
    #endif
      theWindow = theNewWindow;
    };
    GrWindow(CHAR *windowName,INT width=500,INT height=500, INT ReservedColors = 0) {
    #ifdef DEBUG
      SETFUNC("GrWindow::GrWindow(CHAR *,INT,INT)");
    #endif 
      theWindow = grCreateWindow(windowName, width, height, ReservedColors);
    };
    virtual ~GrWindow() {
    #ifdef DEBUG
      SETFUNC("GrWindow::~GrWindow()");
    #endif
      grDestroyWindow(theWindow);
    };
    

    INT AllocateColorTransition(const char *FromColorName, 
                                const char *ToColorName, int NrOfSteps, 
                                int &BaseOffset)

    {
        return grAllocateColorTransition(theWindow, FromColorName, ToColorName,
                                         NrOfSteps, &BaseOffset);
    }

    INT Consistent() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::Consistent()");
    #endif
      return grConsistent(theWindow);
    };
    
    VOID CreateBackground(INT n) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::CreateBackground(INT)");
    #endif
      grCreateBackground(theWindow,n);
    };
    INT SetDrawable(INT n) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::SetDrawable(INT)");
    #endif
      return grSetDrawable(theWindow,n);
    };
    INT BitBlt(INT dst,REAL dst_x,REAL dst_y,INT src,REAL src_x,REAL src_y,
      REAL width,REAL height) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::UsrBitBlt(INT,REAL,REAL,INT,REAL,REAL,REAL,REAL)");
    #endif
      return grUsrBitBlt(theWindow,dst,dst_x,dst_y,src,src_x,src_y,width,height);
    };
    INT DumpBackground(INT n) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::DumpBackground(INT)");
    #endif
      return grDumpBackground(theWindow,n);
    };

    VOID SetTitle(CHAR *windowTitle,CHAR *iconTitle) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetTitle(CHAR *,CHAR *)");
    #endif
      grSetTitle(theWindow,windowTitle,iconTitle);
    };
    virtual VOID SetXaxis(REAL Xmin,REAL Xmax) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetXaxis(REAL,REAL)");
    #endif
      grSetXaxis(theWindow,Xmin,Xmax);
    };
    virtual VOID SetYaxis(REAL Ymin,REAL Ymax) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetYaxis(REAL,REAL)");
    #endif
      grSetYaxis(theWindow,Ymin,Ymax);
    };
    VOID SetZaxis(REAL Zmin,REAL Zmax) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetZaxis(REAL,REAL)");
    #endif
      grSetZaxis(theWindow,Zmin,Zmax);
    };
    VOID SetColorMode(INT colorMode) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetColorMode(INT)");
    #endif
      grSetColorMode(theWindow,colorMode);
    };
    Display *X11Display() {
    #ifdef DEBUG
      SETFUNC("Display *GrWindow::X11Display()");
    #endif
      return grX11Display(theWindow);
    };
    UNSIGNED X11Screen() {
    #ifdef DEBUG
      SETFUNC("UNSIGNED GrWindow::X11Screen()");
    #endif
      return grX11Screen(theWindow);
    };
    UNSIGNED X11Depth() {
    #ifdef DEBUG
      SETFUNC("UNSIGNED GrWindow::X11Depth()");
    #endif
      return grX11Depth(theWindow);
    };
    Visual *X11Visual() {
    #ifdef DEBUG
      SETFUNC("Visual *GrWindow::X11Depth()");
    #endif
      return grX11Visual(theWindow);
    };
    GC X11GC() {
    #ifdef DEBUG
      SETFUNC("GC GrWindow::X11GC()");
    #endif
      return grX11GC(theWindow);
    };
    Window X11RootWindow() {
    #ifdef DEBUG
      SETFUNC("Window GrWindow::X11RootWindow()");
    #endif
      return grX11RootWindow(theWindow);
    };
    Window X11Window() {
    #ifdef DEBUG
      SETFUNC("Window GrWindow::X11Window()");
    #endif
      return grX11Window(theWindow);
    };
    XSizeHints X11XSizeHints() {
    #ifdef DEBUG
      SETFUNC("XSizeHints GrWindow::X11XSizeHints()");
    #endif
      return grX11XSizeHints(theWindow);
    };
    XSetWindowAttributes X11XSetWindowAttributes() {
    #ifdef DEBUG
      SETFUNC("XSetWindowAttributes GrWindow::X11XSetWindowAttributes()");
    #endif
      return grX11XSetWindowAttributes(theWindow);
    };
    XColor X11XColor(INT n) {
    #ifdef DEBUG
      SETFUNC("XColor GrWindow::X11XColor(INT)");
    #endif
      return grX11XColor(theWindow,n);
    };
    XEvent X11XEvent() {
    #ifdef DEBUG
      SETFUNC("XEvent GrWindow::X11XEvent()");
    #endif
      return grX11XEvent(theWindow);
    };
    
    Bool X11CheckMaskEvent(LONG eventMask) {
    #ifdef DEBUG
      SETFUNC("Bool GrWindow::X11CheckMaskEvent(LONG)");
    #endif
      return grX11CheckMaskEvent(theWindow,eventMask);
    };
    VOID X11NextEvent() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11NextEvent()");
    #endif
      grX11NextEvent(theWindow);
    };
    VOID X11WaitUntilExposed() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11WaitUntilExposed()");
    #endif
      grX11WaitUntilExposed(theWindow);
    };
    
    VOID X11WaitUntilUnmapped() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11WaitUntilUnmapped()");
    #endif
      grX11WaitUntilUnmapped(theWindow);
    };
    
    VOID X11SetSpeed(INT s) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11SetSpeed(INT");
    #endif
      grX11SetSpeed(theWindow,s);
    };
    INT X11GetSpeed() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11SetSpeed(INT");
    #endif
      return grX11GetSpeed(theWindow);
    };
    
    VOID X11Flush() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11Flush()");
    #endif
      grX11Flush(theWindow);
    };
    
    VOID X11SelectInput(const LONG inputMask) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11SetectInput(const LONG)");
    #endif
      grX11SelectInput(theWindow,inputMask);
    };
    
    VOID X11SetCursor(const Cursor cursor) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::X11SetCursor(const Cursor)");
    #endif
      grX11SetCursor(theWindow,(unsigned int) cursor);
    };
    
    VOID OpenPostScript(CHAR *postScriptFileName) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::OpenPostScript(CHAR *)");
    #endif
      grOpenPostScript(theWindow,postScriptFileName);
    };
    VOID ClosePostScript() {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::ClosePostScript()");
    #endif
      grClosePostScript(theWindow);
    };

    int GetColor(int Color)
    {
        if (Color >= 0)
            return Color;
        else
            return theWindow->BaseColorIndex - Color - 1;
    }

    double GetWorldX(int X)
    {
        return IX(theWindow, X);
    }

    double GetWorldY(int Y)
    {
        return IY(theWindow, Y);
    }

    VOID SetLengthUnit(INT lu) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetLengthUnit()");
    #endif
      grSetLengthUnit(theWindow,lu);
    };
    INT GetLengthUnit() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::GetLengthUnit()");
    #endif
      return grGetLengthUnit(theWindow);
    };
    VOID SetPaperType(INT pt) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::SetPaperType(INT)");
    #endif
      grSetPaperType(theWindow,pt);
    };
    INT GetPaperType() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::GetPaperType()");
    #endif
      return grGetPaperType(theWindow);
    };
    
    INT NumColors() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::NumColors()");
    #endif
      return grNumColors(theWindow);
    };
    INT MaxX() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::MaxX()");
    #endif
      return grMaxX(theWindow);
    };
    INT MaxY() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::MaxY()");
    #endif
      return grMaxY(theWindow);
    };
    REAL Xmin() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Xmin()");
    #endif
      return grXmin(theWindow);
    };
    REAL Xmax() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Xmax()");
    #endif
      return grXmax(theWindow);
    };
    REAL Ymin() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Ymin()");
    #endif
      return grYmin(theWindow);
    };
    REAL Ymax() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Ymax()");
    #endif
      return grYmax(theWindow);
    };
    REAL Zmin() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Zmin()");
    #endif
      return grZmin(theWindow);
    };
    REAL Zmax() {
    #ifdef DEBUG
      SETFUNC("REAL GrWindow::Zmax()");
    #endif
      return grZmax(theWindow);
    };

    INT LoadFont(INT family,INT weight,INT shape,INT size) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::LoadFont(INT,INT,INT,INT)");
    #endif
      return grLoadFont(theWindow,family,weight,shape,size);
    };
    INT LoadFontName(CHAR *name) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::LoadFontName(CHAR *)");
    #endif
      return grLoadFontName(theWindow,name);
    };
    VOID UnloadFont(INT family,INT weight,INT shape,INT size) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::UnloadFont(INT,INT,INT,INT)");
    #endif
      grUnloadFont(theWindow,family,weight,shape,size);
    };
    VOID UnloadFontName(CHAR *name) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::UnloadFontName(CHAR *)");
    #endif
      grUnloadFontName(theWindow,name);
    };
    INT NoFonts() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::NoFonts()");
    #endif
      return grNoFonts(theWindow);
    };
    INT TopFont() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::TopFont()");
    #endif
      return grTopFont(theWindow);
    };
    INT GetFontNo(INT family,INT weight,INT shape,INT size) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::GetFontNo(INT,INT,INT,INT)");
    #endif
      return grGetFontNo(theWindow,family,weight,shape,size);
    };
    INT SetFont(INT family,INT weight,INT shape,INT size) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::SetFont(INT,INT,INT,INT)");
    #endif
      return grSetFont(theWindow,family,weight,shape,size);
    };
    INT SetHorizontalTextAlignment(INT a) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::SetHorizontalTextAlignment(INT)");
    #endif
      return grSetHorizontalTextAlignment(theWindow,a);
    };
    INT GetHorizontalTextAlignment() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::GetHorizontalTextAlignment()");
    #endif
      return grGetHorizontalTextAlignment(theWindow);
    };
    INT SetVerticalTextAlignment(INT a) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::SetVerticalTextAlignment(INT)");
    #endif
      return grSetVerticalTextAlignment(theWindow,a);
    };
    INT GetVerticalTextAlignment() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::GetVerticalTextAlignment()");
    #endif
      return grGetVerticalTextAlignment(theWindow);
    };
    INT TextWidth(CHAR *s) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::TextWidth(CHAR *)");
    #endif
      return (INT) grGetTextWidth(theWindow,s);
    };
    INT TextHeight() {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::TextHeight()");
    #endif
      return grGetTextHeight(theWindow);
    };
    REAL LeftTextBoundary(REAL x,CHAR *s) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::LeftTextBoundary(REAL ,CHAR *)");
    #endif
      return grUsrLeftTextBoundary(theWindow,x,s);
    };
    REAL RightTextBoundary(REAL x,CHAR *s) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::RightTextBoundary(REAL ,CHAR *)");
    #endif
      return grUsrRightTextBoundary(theWindow,x,s);
    };
    REAL TopTextBoundary(REAL y,CHAR *) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::TopTextBoundary(REAL ,CHAR *)");
    #endif
      return grUsrTopTextBoundary(theWindow,y,s);
    };
    REAL BottomTextBoundary(REAL y,CHAR *) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::BottomTextBoundary(REAL ,CHAR *)");
    #endif
      return grUsrBottomTextBoundary(theWindow,y,s);
    };
#ifdef INLINE_BUG
    INT HorizontalOverlap(REAL x,CHAR *s);
    INT VerticalOverlap(REAL y,CHAR *);
#else
    INT HorizontalOverlap(REAL x,CHAR *s) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::HorizontalOverlap(REAL,CHAR *");
    #endif
      return grUsrHorizontalOverlap(theWindow,x,s);
    };
    INT VerticalOverlap(REAL y,CHAR *) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::VerticalOverlap(REAL,CHAR *");
    #endif
      return grUsrVerticalOverlap(theWindow,y,s);
    };
#endif
    INT HorizontalAdjustTextAlignment(REAL x,CHAR *s) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::HorizontalAdjustTextAlignment(REAL,CHAR *");
    #endif
      return (INT) grUsrHorizontalAdjustTextAlignment(theWindow,x,s);
    };
    INT VerticalAdjustTextAlignment(REAL y,CHAR *) {
    #ifdef DEBUG
      SETFUNC("INT GrWindow::VerticalAdjustTextAlignment(REAL,CHAR *");
    #endif
      return (INT) grUsrVerticalAdjustTextAlignment(theWindow,y,s);
    };
    VOID AdjustTextAlignment(REAL x,REAL y,CHAR *s) {
    #ifdef DEBUG
      SETFUNC("VOID GrWindow::AdjustTextAlignment(REAL,REAL,CHAR *");
    #endif
      grUsrAdjustTextAlignment(theWindow,x,y,s);
    };

    GrWindow &ClearScreen(INT colorNo) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::ClearScreen(INT)");
    #endif
      grClearScreen(theWindow,GetColor(colorNo));
      return *this;
    };
    GrWindow &ClearScreenPS(INT colorNo) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::ClearScreenPS(INT)");
    #endif
      grClearScreenPS(theWindow,GetColor(colorNo));
      return *this;
    };
    GrWindow &operator<<(const GrPoint &p) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrPoint &)");
    #endif
      grUsrDrawPoint(theWindow,p.x,p.y,GetColor(p.color));
      return *this;
    };
    GrWindow &operator<<(const GrPointPS &p) {
    #ifdef DEBUG
      SETFUNC("GrWindow GrWindow::operator<<(GrPointPS &)");
    #endif
      grDrawPointPS(theWindow,p.x,p.y,GetColor(p.color));
      return *this;
    };
#ifdef INLINE_BUG
    GrWindow &operator<<(const GrLine &l);
#else
    GrWindow &operator<<(const GrLine &l) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrLine &)");
    #endif
      grUsrDrawLine(theWindow,l.x1,l.y1,l.x2,l.y2,GetColor(l.color));
      return *this;
    };
#endif
    GrWindow &operator<<(const GrLinePS &l) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrLinePS &)");
    #endif
      grDrawLinePS(theWindow,l.x1,l.y1,l.x2,l.y2,GetColor(l.color));
      return *this;
    };
#ifdef INLINE_BUG
    GrWindow &operator<<(const GrBox &b);
#else
    GrWindow &operator<<(const GrBox &b) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrBox &)");
    #endif
      grUsrDrawBox(theWindow,b.x1,b.y1,b.x2,b.y2,GetColor(b.color));
      return *this;
    };
#endif
    GrWindow &operator<<(const GrBoxPS &b) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrBoxPS &)");
    #endif
      grDrawBoxPS(theWindow,b.x1,b.y1,b.x2,b.y2,GetColor(b.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillBox &b) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillBox &b)");
    #endif
      grUsrDrawFillBox(theWindow,b.x1,b.y1,b.x2,b.y2,GetColor(b.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillBoxPS &b) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillBoxPS &)");
    #endif
      grDrawFillBoxPS(theWindow,b.x1,b.y1,b.x2,b.y2,GetColor(b.color));
      return *this;
    };
    GrWindow &operator<<(const GrEllipticArc &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrEllipticArc &)");
    #endif
      grUsrDrawEllipticArc(theWindow,a.x,a.y,a.a,a.b,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
    GrWindow &operator<<(const GrEllipticArcPS &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrEllipticArcPS &)");
    #endif
      grDrawEllipticArcPS(theWindow,a.x,a.y,a.a,a.b,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillEllipticArc &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillEllipticArc &)");
    #endif
      grUsrDrawFillEllipticArc(theWindow,a.x,a.y,a.a,a.b,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillEllipticArcPS &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillEllipticArcPS &)");
    #endif
      grDrawFillEllipticArcPS(theWindow,a.x,a.y,a.a,a.b,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
    GrWindow &operator<<(const GrEllipse &e) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrEllipse &)");
    #endif
      grUsrDrawEllipse(theWindow,e.x,e.y,e.a,e.b,GetColor(e.color));
      return *this;
    };
    GrWindow &operator<<(const GrEllipsePS &e) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrEllipsePS &)");
    #endif
      grDrawEllipsePS(theWindow,e.x,e.y,e.a,e.b,GetColor(e.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillEllipse &e) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillEllipse &)");
    #endif
      grUsrDrawFillEllipse(theWindow,e.x,e.y,e.a,e.b,GetColor(e.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillEllipsePS &e) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillEllipsePS &)");
    #endif
      grDrawFillEllipsePS(theWindow,e.x,e.y,e.a,e.b,GetColor(e.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillPolygon &p) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillPolygon &)");
    #endif
      grDrawFillPolygon(theWindow,p.Points, p.NrOfPoints, 
                        p.shape, p.mode, GetColor(p.color));
      return *this;
    };
    GrWindow &operator<<(const GrArc &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrArc &)");
    #endif
      grUsrDrawArc(theWindow,a.x,a.y,a.r,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
    GrWindow &operator<<(const GrArcPS &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrArcPS &)");
    #endif
      grDrawEllipticArcPS(theWindow,a.x,a.y,a.r,a.r,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
#ifdef INLINE_BUG
    GrWindow &operator<<(const GrFillArc &a);
#else
    GrWindow &operator<<(const GrFillArc &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillArc &)");
    #endif
      grUsrDrawFillArc(theWindow,a.x,a.y,a.r,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
#endif
    GrWindow &operator<<(const GrFillArcPS &a) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillArcPS &)");
    #endif
      grDrawFillEllipticArcPS(theWindow,a.x,a.y,a.r,a.r,a.a1,a.a2,GetColor(a.color));
      return *this;
    };
#ifdef INLINE_BUG
    GrWindow &operator<<(const GrCircle &c);
#else
    GrWindow &operator<<(const GrCircle &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrCircle &)");
    #endif
      grUsrDrawCircle(theWindow,c.x,c.y,c.r,GetColor(c.color));
      return *this;
    };
#endif
    GrWindow &operator<<(const GrCirclePS &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrCirclePS &)");
    #endif
      grDrawCirclePS(theWindow,c.x,c.y,c.r,GetColor(c.color));
      return *this;
    };
#ifdef INLINE_BUG
    GrWindow &operator<<(const GrFillCircle &c);
#else
    GrWindow &operator<<(const GrFillCircle &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillCircle &)");
    #endif
      grUsrDrawFillCircle(theWindow,c.x,c.y,c.r,GetColor(c.color));
      return *this;
    };
#endif
    GrWindow &operator<<(const GrFillCirclePS &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillCirclePS &)");
    #endif
      grDrawFillCirclePS(theWindow,c.x,c.y,c.r,GetColor(c.color));
      return *this;
    };
    
    GrWindow &operator<<(const GrChar &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrChar &)");
    #endif
      grUsrDrawChar(theWindow,c.x,c.y,c.c,GetColor(c.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillChar &c) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillChar &)");
    #endif
      grUsrDrawFillChar(theWindow,c.x,c.y,c.c,c.fg,GetColor(c.bg));
      return *this;
    };
    GrWindow &operator<<(const GrString &s) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrString &)");
    #endif
      grUsrDrawString(theWindow,s.x,s.y,s.s,GetColor(s.color));
      return *this;
    };
    GrWindow &operator<<(const GrFillString &s) {
    #ifdef DEBUG
      SETFUNC("GrWindow &GrWindow::operator<<(GrFillString &)");
    #endif
      grUsrDrawFillString(theWindow,s.x,s.y,s.s,s.fg,GetColor(s.bg));
      return *this;
    };
};

#endif
