/*      file     : parallel/THostInfo.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: THostInfo.C,v 1.9 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <string.h>

#include "TBaseMessage.h"
#include "THostInfo.h"

// ***DEFINES************************************************************************* 

#define HOSTINFO_DEFAULT_MAX_CPN        1
#define HOSTINFO_DEFAULT_CPN            0
#define HOSTINFO_DEFAULT_FLAGS          0
#define HOSTINFO_DEFAULT_PERFORMANCE 1000.0

// ***FUNCTIONS*********************************************************************** 

bool
operator == (const THostInfo &Host1, const THostInfo &Host2)

{
    return Host1.ID == Host2.ID;
}

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

THostInfo::THostInfo(const char *theName)

{
    _Init();
    Name = theName;
}

// =================================================================================== 

THostInfo::THostInfo(TBaseMessage *aMessage)

{
    _Init();
    if (!Decode(aMessage))
        Fail("cannot decode from message");
}

// =================================================================================== 

bool
THostInfo::Decode(TBaseMessage *aMessage)

{
    return (aMessage->Decode(ID) AND
            Name.Decode(*aMessage) AND
            ArchName.Decode(*aMessage) AND
            aMessage->Decode(ArchCode) AND
            aMessage->Decode(Performance) AND
            aMessage->Decode(MaxNrOfCPN) AND
            aMessage->Decode(NrOfCPN) AND
            aMessage->Decode(DebugMask) AND
            aMessage->Decode(Flags));
}

// =================================================================================== 

bool
THostInfo::Encode(TBaseMessage *aMessage) const

{
    return (aMessage->Encode(ID) AND
            Name.Encode(*aMessage) AND
            ArchName.Encode(*aMessage) AND
            aMessage->Encode(ArchCode) AND
            aMessage->Encode(Performance) AND
            aMessage->Encode(MaxNrOfCPN) AND
            aMessage->Encode(NrOfCPN) AND
            aMessage->Encode(DebugMask) AND
            aMessage->Encode(Flags));
}

// =================================================================================== 

TString
THostInfo::GetClassName(void) const

{
    return "THostInfo";
}

// =================================================================================== 

TString
THostInfo::GetInstanceName(void) const

{
    char Line[512];

    sprintf(Line, "ID:%lX Name:'%s' Arch:'%s' Perf:%.0f MaxCPN:%d CPN:%d",
            ID, (char *) GetShortName(), ArchName(), Performance, MaxNrOfCPN, NrOfCPN);
    return Line;
}

// =================================================================================== 

TString
THostInfo::GetShortName(void) const

{
    char  name[512];
    char *pos;

    strcpy(name, Name);
    if ((pos = strchr(name, '.')))
        *pos = 0;
    return name;
}

// =================================================================================== 

void
THostInfo::_Init(void)

{
    ID = INVALID_ID;
    MaxNrOfCPN = HOSTINFO_DEFAULT_MAX_CPN;
    NrOfCPN = HOSTINFO_DEFAULT_CPN;
    Performance = HOSTINFO_DEFAULT_PERFORMANCE;
    ArchName = "unknown";
    Name = "unknown";
    Flags = HOSTINFO_DEFAULT_FLAGS;
    DebugMask = DM_PARALLEL | DM_MSG_CONTROL;
}

// =================================================================================== 

bool
THostInfo::SameName(const char *aName) const

{
    char    *Pos;
    char    *OtherPos;
    char     Name1[128];
    char     Name2[128];

    Pos = strchr(Name(), '.');
    OtherPos = strchr(aName, '.');
    if ((Pos == 0 AND OtherPos == 0) OR (Pos != 0 AND OtherPos != 0))
        return (StrCmp(Name, aName, STRCMP_IGNORE_CASE) == 0);
    strcpy(Name1, Name());
    strcpy(Name2, aName);
    if (Pos)
        *strchr(Name1, '.') = 0;
    if (OtherPos)
        *strchr(Name2, '.') = 0;
    return (StrCmp(Name1, Name2, STRCMP_IGNORE_CASE) == 0);
}

// =================================================================================== 
#undef ParentClass
