/*      file     : parallel/TMessageRecipient.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMessageRecipient.C,v 1.7 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TDebug.h>
#include <TEnvironment.h>
#include "TBaseMessage.h"
#include "TEventHandler.h"
#include "TMessageRecipient.h"

// ***METHODS************************************************************************* 

TMessageRecipient::TMessageRecipient(void)

{
    EventHandler = 0;
}

// =================================================================================== 

bool
TMessageRecipient::HandleEvent(TEventID, TBaseMessage *, int &)

{
    return TRUE;
}

// =================================================================================== 

bool
TMessageRecipient::HandleEventMessage(TBaseMessage *&aMessage, bool &ExitLoop)

{
    TEventID EventID;
    int      Handled = TRUE;

    while ((Handled != FALSE) AND aMessage->EventPending(EventID))
    {
        Handled = TRUE;
        if (EventHandler)
        {
            if (!EventHandler->HandleEvent(EventID, aMessage, Handled))
            {
                Debug.Message("TMessageRecipient: error while handling event with ID %ld",
                              EventID);
                return FALSE;
            }
            // Debug.Message("Handled: %d", Handled);
        }
        else
            if (!HandleEvent(EventID, aMessage, Handled))
            {
                Debug.Message("TMessageRecipient: error while handling event with ID %ld", 
                              EventID);
                return FALSE;
            }
        switch (Handled)
        {
        case FALSE:
            Debug.Message("TMessageRecipient: event with ID %ld not handled", EventID);
            aMessage->Flush();
            break;

        case EXIT_LOOP:
            // Debug.Message("TMessageRecipient: previous event caused exit of loop");
            ExitLoop = TRUE;
        }
    }
    return TRUE;
}

// =================================================================================== 

bool
TMessageRecipient::HandleMessage(TBaseMessage *&aMessage, bool &ExitLoop)

{
    if (aMessage->GetFlags() & MSG_FLAG_EVENT)
        return HandleEventMessage(aMessage, ExitLoop);
    Debug.Message("TMessageRecipient::HandleMessage: message (%s) not handled", 
                  (char *) aMessage->GetInstanceName());
    return FALSE;
}

// =================================================================================== 

void
TMessageRecipient::SetEventHandler(const TEventHandlerPrimitive *theEventHandler)

{
    EventHandler = theEventHandler;
}

// =================================================================================== 
