/*      file     : parallel/TParallel.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TParallel.h,v 1.22 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TPARALLEL_H
#define TPARALLEL_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TDLList.h>

#include <ParallelTypes.h>
#include <TParallelControl.h>
#include <TMessageRecipient.h>
#include <TDestination.h>

// ***FORWARDS************************************************************************

class TCPNInfo;
class THostInfo;
class TNetwork;
class TParallelControl;
class TString;

// ***CLASSES************************************************************************* 

/** High level parallel control class.
 */

class TParallel : public TBaseObject, public TMessageRecipient

{
 public:

    TParallel(void);

    virtual ~TParallel(void);

    bool
        AddAllCPN(void);
    bool
        AddCPN(THostInfo *aHostInfo, int NrOfInstances);
    bool
        AddMaxCPN(THostInfo *aHostInfo);
    bool
        Activate(void);
    TCPNInfo *
        GetCPNInfoByID(TCPNID SearchID) const;
    const TDLList<TCPNInfo> *
        GetCPNInfoList(void) const;
    inline int
        GetCurrentNrOfCPN(void) const;
    virtual TString
        GetInstanceName(void) const;
    inline TCPNID 
        GetLocalCPNID(void) const;
    inline const TCPNInfo *
        GetLocalCPNInfo(void) const;
    int
        GetMaxNrOfCPN(void) const;
    inline TCPNID 
        GetParentCPNID(void) const;
    inline TTopology *
        GetPhysicalTopology(void);
    inline long
         GetUniqueID(long OtherID);
    inline bool
        HasQuit(void) const;
    bool
        IsMaster(void) const;
    bool
        Quit(void) const;
    inline bool
        RegisterTopologyUser(TTopologyUser *theUser);
    bool
        ShowCPN(void) const;
    bool
        ShowHosts(void) const;
    bool
        Start(void);
    inline bool
         Synchronize(TDestination Destination, bool ExitLoop = FALSE);

 private:


    bool
        BroadcastConfiguration(void);
    bool
        Deactivate(void);
    virtual TString
        GetClassName(void) const;
    virtual bool
        HandleEvent(TEventID theEventID, TBaseMessage *theMessage, int &Handled);
    void
        _Init(void);
    bool
        ReceiveConfiguration(TBaseMessage &aMessage, int &Handled);

    TParallelControl *Control;
    TNetwork         *Network;

    int  ForcedNrOfCPN;
    bool BroadcastConfigActive;
};

// ***EXTERNALS*********************************************************************** 

extern TParallel *Parallel;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TParallel.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
