/*      file     : parallel/TTopology.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTopology.C,v 1.6 2005/03/25 11:06:21 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "TParallel.h"
#include "TCPNInfo.h"

#include "TTopology.h"

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TTopology.i.C"
#endif

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TTopology::TTopology(const TDLList<TCPNInfo> *theList)

{
    List = theList? theList : Parallel->GetCPNInfoList();
    Mutable = new TTopologyMutable;
    Mutable->MyRank = INVALID;
}

// ===================================================================================

TTopology::~TTopology(void)

{
    DeleteObject(Mutable);
}

// ===================================================================================

bool
TTopology::DetermineRank(void) const

{
    TCPNInfoEnum CInfo;
    int          i;
    int          NrOfCPN;

    if (IsTopologyCurrent())
        return TRUE;
    i = 0;
    NrOfCPN = (int) GetList()->GetNrOfElements();
    debug(Print(DM_PARALLEL, "determine rank for %d CPN", NrOfCPN));
    if (!Mutable->SortedInfos.Resize(NrOfCPN))
    {
        Error("cannot resize CPN info array");
        return FALSE;
    }
    CInfo.WalksForwardThrough(GetList());
    while (CInfo())
    {
        Mutable->SortedInfos.Get(i) = (TCPNInfo *) CInfo;
        CInfo++;
        i++;
    }
    qsort((void *) Mutable->SortedInfos.GetItemPtr(), NrOfCPN, sizeof(TCPNInfo *), 
          (int (*) (const void *, const void *)) CompareCPNInfosByHostAndID);
    Mutable->MyRank = INVALID;
    i = 0;
    while (i < NrOfCPN /* AND Mutable->MyRank == INVALID */)
    {
        debug(Print(DM_PARALLEL, "sorted CPN: %s", 
                    (char *) Mutable->SortedInfos.Get(i)->GetInstanceName()));
        if (Mutable->SortedInfos.Get(i) == Parallel->GetLocalCPNInfo())
            Mutable->MyRank = i;
        i++;
    }
    if (Mutable->MyRank == INVALID)
    {
        Error("cannot determine my rank");
        return FALSE;
    }
    MarkTopologyCurrent();
    return TRUE;
}

// ===================================================================================


TString
TTopology::GetClassName(void) const

{
    return "TTopology";
}

// =================================================================================== 

const TCPNInfo *
TTopology::GetCPNInfoByRank(int aRank) const

{
    if (!DetermineRank())
        return 0;
    return Mutable->SortedInfos.Get(GetNormalizedRank(aRank));
}

// =================================================================================== 

TString
TTopology::GetInstanceName(void) const

{
    char Line[64];

    if (List)
        sprintf(Line, "%ld CPN", List->GetNrOfElements());
    else
        sprintf(Line, "? CPN");
    return Line;
}

// =================================================================================== 

const TDLList<TCPNInfo> *
TTopology::GetList(void) const

{
    return List;
}

// =================================================================================== 

const TCPNInfo *
TTopology::GetNext(const TCPNInfo *) const

{
    int MyRank;

    if ((MyRank = GetRank()) == INVALID)
        return 0;
    return GetCPNInfoByRank(GetNextRank(MyRank));
}

// =================================================================================== 

const TCPNInfo *
TTopology::GetPrev(const TCPNInfo *) const

{
    int MyRank;

    if ((MyRank = GetRank()) == INVALID)
        return 0;
    return GetCPNInfoByRank(GetPrevRank(MyRank));
}

// =================================================================================== 

int
TTopology::GetRank(const TCPNInfo *aCPNInfo) const

{
    int             Rank;
    int             NrOfCPN;

    if (!DetermineRank())
        return INVALID;
    if (aCPNInfo == 0)
        return Mutable->MyRank;
    
    Rank = 0;
    NrOfCPN = (int) GetList()->GetNrOfElements();
    while (Rank < NrOfCPN)
    {
        if (Mutable->SortedInfos.Get(Rank) == aCPNInfo)
            return Rank;
        Rank++;
    }
    if (aCPNInfo == 0)
        Warning("cannot determine rank for local CPN Info");
    return INVALID;
}

// =================================================================================== 

const TCPNInfo *
TTopology::GetRelative(int Displacement) const

{
    return GetCPNInfoByRank(GetRank() + Displacement);
}

// =================================================================================== 

TTopologyType
TTopology::GetType(void) const

{
    return SimpleTopology;
}

// =================================================================================== 
#undef ParentClass

