/*      file     : parbalance/TTransferLoadInfo.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTransferLoadInfo.h,v 1.4 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TTRANSFERLOADINFO_H
#define TTRANSFERLOADINFO_H

// ***INCLUDES************************************************************************ 

#include <TDLList.h>
#include <TTimeStep.h>
#include <TParallelListObject.h>
#include <TMigrationContext.h>

// ***FORWARDS************************************************************************

class TClassInfo;

// ***TYPES*************************************************************************** 

enum TTransferLoadMode

{
    NoTransfer,         // common for send + receive
    TransferSelected,   // send modes
    WaitForAcknowledge,
    WaitUntilCPNReady,
    SendLoad,
    WaitForConfirm,     // receive modes
    ReceiveLoad
};

// ***CLASSES************************************************************************* 

class TTransferLoadInfo : public TParallelListObject

{
public:

    TTransferLoadInfo(TParallelID theParallelID);

    void
    Acknowledge(void);

    void
    Cancel(void);

    void
    Confirm(void);

    virtual TString
    GetInstanceName(void) const;

    inline double
    GetTransferLoad(void) const;

    inline const TTimeStep &
    GetCollectTimeStep(void) const;

    inline TMigrationContext *
    GetMigrationContext(void) const;

    inline TTransferLoadMode
    GetMode(void) const;

    inline int
    GetNrOfMissingAcks(void) const;

    inline int
    GetNrOfNonReadyCPN(void) const;

    inline const TTimeStep &
    GetTransferTimeStep(void) const;

    void
    Ready(void);

    void
    Reset(void);

    inline void
    Set(const TTimeStep &theCollectTimeStep, double theTransferLoad);

    inline void
    SetMode(TTransferLoadMode theMode);

    inline void
    SetReceiveTimeStep(const TTimeStep &theTransferTimeStep);

    inline void
    SetSendTimeStep(const TTimeStep &theTransferTimeStep, TMigrationContext *theContext);

    static TClassInfo *ClassInfo;

private:

    virtual TString
    GetClassName(void) const;

    TMigrationContext *MigrationContext;

    TCPNID             RemoteCPNID;
    TTimeStep          CollectTimeStep;
    TTimeStep          TransferTimeStep;
    double             TransferLoad;
    TTransferLoadMode  Mode;
    int                NrOfMissingAcks;
    int                NrOfNonReadyCPN;
};


// ***TYPES*************************************************************************** 

typedef TDLList< TTransferLoadInfo > TTransferLoadInfoList;
typedef TDLListEnum< TTransferLoadInfo > TTransferLoadInfoEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TTransferLoadInfo.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
