/*      file     : parstat/TStatManager.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TStatManager.h,v 1.7 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSTATMANAGER_H
#define TSTATMANAGER_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TMessageRecipient.h>
#include <TObjectHandler.h>

// ***FORWARDS************************************************************************

class TAggregationResult;
class TTopology;
class TStatHandler;
class TStatHandlerInfo;

#include "TStatObject.forward.h"
//#include "TSamplingDescription.forward.h"
#include "TSamplingDescription.h"
#include "TAggregationInfo.forward.h"

// ***TYPES*************************************************************************** 

typedef TABTree < TStatHandlerInfo > THandlerTable;

// ***CLASSES************************************************************************* 

class TStatManager : public TBaseObject , public TMessageRecipient, 
                     public TObjectHandler

{
public:

    TStatManager(void);
    virtual ~TStatManager(void);

    bool
    Activate(void);

    bool
    BroadcastSamplingDescription(TSamplingDescription *Desc);

    bool
    BroadcastSamplingDescriptions(void);

    bool
    CancelAggregation(TAggregationInfo *AggregationInfo);

    bool
    ExecuteSubTimeStep(TTimeStep &TimeStep);

    bool
    ModifySampling(TSamplingDescription *Description);

    bool
    RegisterAggregation(TAggregationInfo *AggregationInfo);

    bool
    RegisterSamplingHandler(TClassID StatObjectClassID, TStatHandler *theHandler);

    bool
    Run(TTopology *theTopology);

    bool
    StartSampling(TSamplingDescription *Description, bool Brodcast = FALSE);

    bool
    StopSampling(TSamplingDescription *Description);

private:

    bool
    Collect(TSamplingDescription *Desc, TTimeStep &TimeStep);
 
    bool
    DeleteSamplingDescription(TBaseMessage *Message);

    TAggregationInfo *
    GetAggregationInfo(TParallelID SamplingID);

    virtual TString
    GetClassName(void) const;

    TStatHandler *
    GetHandler(TClassID StatObjectClassID);

    virtual TString
    GetInstanceName(void) const;

    TSamplingDescription *
    GetSamplingDescription(TParallelID SamplingID);

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);

    virtual bool
    HandleEvent(TEventID EventID, TBaseMessage *Message, int &Handled);

    bool
    MergeStatObjects(TSamplingDescription *Description,
                     TAggregationResult *Result, TStatObjectList *NewList);

    bool
    ProcessStatObjects(TSamplingDescription *Desc, TAggregationInfo *Info,
                       TTimeStep &TimeStep, TStatObjectList *StatObjects);

    bool
    ReceiveSamplingDescriptions(TBaseMessage *Message);

    bool
    ReceiveStatObjects(TBaseMessage *Message);

    bool
    ResetPreviouslyCollectedStats(const TTimeStep &TimeStep);

    bool
    Sample(TSamplingDescription *Desc, TTimeStep &TimeStep);

    TTopology                *Topology;
    TSamplingDescriptionList *SamplingDescriptions;
    TAggregationInfoTable    *AggregationInfos;
    THandlerTable            *Handlers;
};

// ***EXTERNALS*********************************************************************** 

extern TStatManager *StatManager;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
