/*      file     : parstat/TStatObject.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        objects  : TStatObject
        RCS      : $Id: TStatObject.h,v 1.4 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSTAT_H
#define TSTAT_H

/* ***INCLUDES************************************************************************ */

#include <TDLList.h>
#include <TTimeStep.h>
#include <TParallelListObject.h>

/* ***DEFINES************************************************************************* */

#define INVALID_MEAN (-1)

/* ***CLASSES************************************************************************* */

/** Base class to all statistical infos. */

class TStatObject : public TParallelListObject

{
public:


    TStatObject(TClassInfo *theClassInfo, TParallelID theParallelID);

    virtual bool
    Accumulate(const TStatObject *StatObject);

    bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual bool
    Evaluate(void);

    virtual TString
    GetClassName(void) const;

    inline double
    GetFraction(void) const;

    inline long
    GetNrOfSamples(void) const;

    virtual TString
    GetInstanceName(void) const;

    inline const TTimeStep &
    GetTimeStep(void) const;

    inline TTimeStep &
    GetTimeStep(void);

    virtual void
    Reset(void);

    inline void
    SetFraction(double theFraction);

    inline void
    SetNrOfSamples(long theNrOfSamples);

    inline void
    SetTimeStep(const TTimeStep &theTimeStep);

protected:
    
    TTimeStep TimeStep;
    long      NrOfSamples;
    double    Fraction;
};

// ***TYPES*************************************************************************** 

typedef TDLList < TStatObject > TStatObjectList;
typedef TDLListEnum < TStatObject > TStatObjectEnum;

// ***PROTOTYPES********************************************************************** 
#ifdef __cplusplus
extern "C" {
#endif

    double
        SecureDiv(double a, double b);

#ifdef __cplusplus
}
#endif
// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TStatObject.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
