/*      file     : partools/TIncidenceInfo.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TIncidenceInfo.C,v 1.8 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <string.h>
#include <TBaseStream.h>
#include <TTransferInfo.h>

#include "TEdge.h"

#include "TIncidenceInfo.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TIncidenceInfo::ClassInfo = 0;

// ***FUNCTIONS*********************************************************************** 

int
CompareFromNodeIDs(TEdge **Edge1, TEdge **Edge2)

{
    return CompareLong((*Edge1)->GetFromNodeID(), (*Edge2)->GetFromNodeID());
}

// =================================================================================== 

int
CompareToNodeIDs(TEdge **Edge1, TEdge **Edge2)

{
    return CompareLong((*Edge1)->GetToNodeID(), (*Edge2)->GetToNodeID());
}

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TIncidenceInfo.i.C"
#endif

// =================================================================================== 
#define ParentClass TParallelObject
// =================================================================================== 

TIncidenceInfo::TIncidenceInfo(TClassInfo *ClassInfo, TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    NrOfInEdges = 0;
    NrOfOutEdges = 0;
    InEdges = 0;
    OutEdges = 0;
}

// =================================================================================== 

TIncidenceInfo::~TIncidenceInfo(void)

{
    DeleteObjectArray(InEdges);
    DeleteObjectArray(OutEdges);
}

// =================================================================================== 

bool
TIncidenceInfo::Allocate(void)

{
    InEdges = new TParallelID [NrOfInEdges];
    OutEdges = new TParallelID [NrOfOutEdges];
    return TRUE;
}

// =================================================================================== 

bool
TIncidenceInfo::AssignEdges(int theNrOfInEdges, TParallelID  *theInEdges,
                            int theNrOfOutEdges, TParallelID *theOutEdges)

{
    NrOfInEdges = theNrOfInEdges;
    NrOfOutEdges = theNrOfOutEdges;
    if (!Allocate())
        return FALSE;
    if (NrOfInEdges > 0)
        memcpy(InEdges, theInEdges, NrOfInEdges * sizeof(TParallelID));
    if (NrOfOutEdges > 0)
        memcpy(OutEdges, theOutEdges, NrOfOutEdges * sizeof(TParallelID));

    //    Print("assign");

    //    return Sort();
    return TRUE;
}

// =================================================================================== 

bool
TIncidenceInfo::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
    {
        if (!(Stream->Decode(NrOfInEdges) AND
              Stream->Decode(NrOfOutEdges)))
            return FALSE;
        if (!Allocate())
            return FALSE;
        if (NrOfInEdges > 0)
            if (!Stream->Decode(InEdges[0], NrOfInEdges))
                return FALSE;
        if (NrOfOutEdges > 0)
            if (!Stream->Decode(OutEdges[0], NrOfOutEdges))
                return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TIncidenceInfo::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)     
    {
        if (!(Stream->Encode(NrOfInEdges) AND
              Stream->Encode(NrOfOutEdges)))
            return FALSE;
        if (NrOfInEdges > 0)
            if (!Stream->Encode(InEdges[0], NrOfInEdges))
                return FALSE;
        if (NrOfOutEdges > 0)
            if (!Stream->Encode(OutEdges[0], NrOfOutEdges))
                return FALSE;
    }
    return TRUE;    
}

// =================================================================================== 

TString
TIncidenceInfo::GetClassName(void) const

{
    return "TIncidenceInfo";
}

// =================================================================================== 

TString
TIncidenceInfo::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, " (%s) In:%d Out:%d", 
            (char *) ParentClass::GetInstanceName(), NrOfInEdges, NrOfOutEdges);
    return Line;
}

// =================================================================================== 
/*
bool
TIncidenceInfo::IsSimpleConnection(void)

{
    if (NrOfInEdges == 1 AND NrOfOutEdges == 1)
        return TRUE;
    if (NrOfInEdges != 2 OR NrOfOutEdges != 2)
        return FALSE;
    return ((InEdges[0]->GetFromNodeID() == OutEdges[0]->GetToNodeID() AND
             InEdges[1]->GetFromNodeID() == OutEdges[1]->GetToNodeID()) OR
            (InEdges[0]->GetFromNodeID() == OutEdges[1]->GetToNodeID() AND
             InEdges[1]->GetFromNodeID() == OutEdges[0]->GetToNodeID()));
}
*/
// =================================================================================== 

bool
TIncidenceInfo::Sort(void)

{
    if (InEdges)
        qsort((void *) InEdges, NrOfInEdges, sizeof(TParallelID),
              (int (*) (const void *, const void *)) CompareParallelIDs);
    if (OutEdges)
        qsort((void *) OutEdges, NrOfOutEdges, sizeof(TParallelID),
              (int (*) (const void *, const void *)) CompareParallelIDs);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
