/*      file     : partools/TNode.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TNode.C,v 1.11 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>
#include <TObjectManager.h>

#include "TTransferInfo.h"
#include "TIncidenceInfo.h"
#include "TClassInfo.h"

#include "TEdge.h"
#include "TNode.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TNode::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TNode.i.C"
#endif

// =================================================================================== 
#define ParentClass TContainerObject
// =================================================================================== 

TNode::TNode(TClassInfo *theClassInfo, TParallelID theParallelID, 
             TInfoID theDefaultIncidenceInfoID) :

ParentClass(theClassInfo, theParallelID)

{
    DefaultIncidenceInfoID = theDefaultIncidenceInfoID;
}

// =================================================================================== 

bool
TNode::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        return Stream->Decode(DefaultIncidenceInfoID);
    else
        return TRUE;
}

// =================================================================================== 

bool
TNode::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        return Stream->Encode(DefaultIncidenceInfoID);
    else
        return TRUE;
}

// =================================================================================== 

TString
TNode::GetClassName(void) const

{
    
    return "TNode";
}

// =================================================================================== 

bool
TNode::GetDependantContainers(int &NrOfRefs, TContainerReference *References) 

{
    TIncidenceInfo *Info;
    int             i;

    if ((Info = (TIncidenceInfo *) GetInfo(DefaultIncidenceInfoID, TRUE, TRUE)) == 0)
    {
        Error("GetDependantContainers: cannot retrieve default incidence info");
        return FALSE;
    }
    if (Info->GetNrOfIncomingEdges() + Info->GetNrOfOutgoingEdges() > MAX_CONTAINER_DEPS)
    {
        Error("Number of dependants %d exceeds maximum MAX_CONTAINER_DEPS=%d", 
              Info->GetNrOfIncomingEdges() + Info->GetNrOfOutgoingEdges(), MAX_CONTAINER_DEPS);
        return FALSE;
    }
    NrOfRefs = 0;
    for (i = 0; i < Info->GetNrOfIncomingEdges(); i++)
    {
        References[NrOfRefs].ClassID = TEdge::ClassInfo->GetID();
        References[NrOfRefs].ParallelID = Info->GetIncomingEdge(i)->GetParallelID();
        NrOfRefs++;
    }
    for (i = 0; i < Info->GetNrOfOutgoingEdges(); i++)
    {
        References[NrOfRefs].ClassID = TEdge::ClassInfo->GetID();
        References[NrOfRefs].ParallelID = Info->GetOutgoingEdge(i)->GetParallelID();
        NrOfRefs++;
    }
    return TRUE;
}

// =================================================================================== 

double
TNode::GetLoad(TCPNID LocalCPNID, TInfoID AdjacenceInfoID, TInfoID LocationInfoID)

{
    TIncidenceInfo *Info;
    int             i;
    double          Load;

    Load = 0.0;
    if (AdjacenceInfoID != INVALID)
    {
        if ((Info = (TIncidenceInfo *) GetInfo(DefaultIncidenceInfoID, TRUE, TRUE)) == 0)
        {
            Error("GetLoad: cannot retrieve default incidence info");
            return FALSE;
        }       
        for (i = 0; i < Info->GetNrOfIncomingEdges(); i++)
            Load += 0.5 * Info->GetIncomingEdge(i)->GetLoad(LocalCPNID, AdjacenceInfoID,
                                                            LocationInfoID);
        for (i = 0; i < Info->GetNrOfOutgoingEdges(); i++)
            Load += 0.5 * Info->GetOutgoingEdge(i)->GetLoad(LocalCPNID, AdjacenceInfoID,
                                                            LocationInfoID);
    }
    return Load + ParentClass::GetLoad(LocalCPNID, AdjacenceInfoID, LocationInfoID);
}

// =================================================================================== 

TNode *
TNode::GetPtr(TParallelID ID)

{
    return (TNode *) ClassInfo->GetObjectPtr(ID);
}

/* =================================================================================== */

TNode *
TNode::LoadInstance(TParallelID NodeID, TInfoMask LocalInfoMask)

{
    TNode *Node;

    if ((Node = GetPtr(NodeID)) == 0)
        Node = (TNode *) ObjectManager->CreateInstance(ClassInfo, NodeID,
                                                       LocalInfoMask);
    return Node;
}

// =================================================================================== 
#undef ParentClass
