/*      file     : partools/TParallelObject.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TParallelObject.h,v 1.14 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TPARALLELOBJECT_H
#define TPARALLELOBJECT_H

// ***INCLUDES************************************************************************ 

#include <malloc.h>

#include <TBaseObject.h>
#include <TDLList.h>

#include <ParallelTypes.h>
#include <ParToolsTypes.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TClassInfo;
class TContainerObject;
class TDistributedRange;
class TParser;
class TStream;
class TTimeStep;
class TTransferInfo;

// #include "TBoundary.forward.h"

// ***DEFINES************************************************************************* 

#define PO_FLAG_FIRST_FREE_FLAG BO_FLAG_FIRST_FREE_FLAG
#define PO_CC_PARALLEL_ID       0
#define PO_CC_FISRT_FREE_CODE   1

#define MAX_HEAPOBJECT_SIZE 1024
#define MAX_HEAPOBJECTS     256

// ***CLASSES************************************************************************* 

class TParallelObject : public TBaseObject

{
public:

    TParallelObject(TClassInfo *ClassInfo, TParallelID ParallelID);
    TParallelObject(TClassInfo *ClassInfo, TBaseStream *Stream, 
                    TTransferInfo *TransferInfo);
    virtual ~TParallelObject(void);

    virtual bool
    Decode(TBaseStream *Stream, TTransferInfo *);

#ifndef NO_FAST_HEAP
    void
    operator delete(void *p, size_t size);
#endif

    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *) const;

    virtual int
    Compare(TBaseObject *OtherObject, int CompareCode) const;

    virtual bool
    DisposeRange(TDistributedRange *);

#ifndef NO_FAST_HEAP
    static void
    DumpHeapManagerStat(void);
#endif

    virtual bool
    GetBoundaries(TTimeStep &TimeStep, TDLListPrimitive *Boundaries);

    TClassID
    GetClassID(void) const;

    inline TClassInfo *
    GetClassInfo(void);

    inline const TClassInfo *
    GetClassInfo(void) const;

    virtual TCPNID
    GetCPNID(void) const;

    inline TContainerObject *
    GetContainer(void) const;

    virtual TParallelObject *
    GetInfo(TInfoID InfoID, bool Create = FALSE, bool Load = FALSE) const;

    virtual TString
    GetInstanceName(void) const;

    virtual double
    GetLoad(TCPNID LocalCPNID, TInfoID AdjacenceInfoID, TInfoID LocationInfoID);

    inline TParallelID
    GetParallelID(void) const;

#ifndef NO_FAST_HEAP
    static void
    InitHeapManager(void);
#endif

    virtual bool
    IsInRange(const TDistributedRange *Range) const;

    virtual bool
    IsDistributable(void) const;

#ifndef NO_FAST_HEAP
    void *
    operator new(size_t size);
#endif

    virtual bool
    ReadEvent(TStream *Stream);

    void
    Register(void);

    void
    SetParallelID(TParallelID theParallelID);

    bool
    Write(TBaseStream *Stream);

    void
    Unregister(void);

protected:

    virtual TString
    GetClassName(void) const;

private:

    TParallelID       ParallelID;
    TClassInfo       *ClassInfo;
    TContainerObject *Container;

};

// ***PROTOTYPES**********************************************************************

int
CompareParallelIDs(TParallelID *ID1, TParallelID *ID2);

int
CompareParallelObjectsByParallelID(TParallelObject *Object1, TParallelObject *Object2);

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TParallelObject.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
