/*      file     : xparallel/TTopologyWindow.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTopologyWindow.h,v 1.7 2005/03/25 17:01:28 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TTOPOLOGYWINDOW_H
#define TTOPOLOGYWINDOW_H

// ***INCLUDES************************************************************************ 

#include <TLocation.h>
#include <TBaseObject.h>
#include <TStatHandler.h>

// ***FORWARDS************************************************************************

class TAggregationInfo;
class TSamplingDescription;
class TTimeStep;
class TTopology;
class TWindow;

// ***CLASSES************************************************************************* 

class TTopologyWindow : public TBaseObject, public TStatHandler

{
public:

    TTopologyWindow(TTopology *theTopology);
    virtual ~TTopologyWindow(void);

    bool
    ExecuteSubTimeStep(TTimeStep &);

private:

    bool
    CheckOpenWindow(void);

    virtual bool
    Draw(void);

    bool
    DrawLinks(void);

    virtual TString
    GetClassName(void) const;

    bool
    HandleExtendedLoadResult(TAggregationInfo *Info, TTimeStep &TimeStep);

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    TTopology            *Topology;

    TWindow              *Window;
    TString               Name;
    int                   Width, Height;
    int                   IdleTimeColor;
    int                   MiscTimeColor;
    int                   AppTimeColor;
    int                   StatTimeColor;
    int                   BackgroundColor;
    int                   DrawFlags;

    int                   SamplingInterval;

    TSamplingDescription *LoadSamplingDescription;
    TAggregationInfo     *LoadAggregationInfo;
    bool                  StatChanged;
    bool                  FrameAvailable;
    TRectangle            Frame;
    double                Border;
    double                NodeRadius;
    double                ArrowWidth;
    double                BoundaryArrowWidth;
    double                RelativeNodeRadius;
    double                RelativeArrowWidth;
    double                RelativeBndArrowWidth;
    double                ApplicationLoad;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
